import pytest
from autocoder.common import AutoCoderArgs


class TestAutoCoderArgsDefaults:
    """Test all default values of AutoCoderArgs"""

    def test_all_field_defaults(self):
        """Test all field default values by creating instance without parameters"""
        args = AutoCoderArgs()
        
        # Test basic fields
        assert args.request_id is None
        assert args.source_dir == "."
        assert args.git_url is None
        assert args.target_file is None
        assert args.query is None
        assert args.template is None
        assert args.project_type is None
        assert args.execute is None
        assert args.package_name == ""
        assert args.script_path == ""
        
        # Test model fields
        assert args.model == ""
        assert args.chat_model == ""
        assert args.model_max_length == 2000
        assert args.model_max_input_length == 6000
        assert args.vl_model == ""
        assert args.sd_model == ""
        assert args.emb_model == ""
        assert args.code_model == ""
        assert args.generate_rerank_model == ""
        assert args.context_prune_model == ""
        assert args.conversation_prune_model == ""
        assert args.inference_model == ""
        assert args.system_prompt == ""
        assert args.planner_model == ""
        assert args.voice2text_model == ""
        assert args.text2voice_model == ""
        assert args.commit_model == ""
        
        # Test index fields
        assert args.skip_build_index is False
        assert args.skip_filter_index is False
        assert args.index_model == ""
        assert args.index_filter_model == ""
        assert args.index_filter_model_max_input_length == 50*1024
        assert args.index_model_max_length == 0
        assert args.index_model_max_input_length == 0
        assert args.index_model_anti_quota_limit == 0
        
        # Test agentic fields
        assert args.enable_agentic_filter is False
        assert args.enable_agentic_edit is True
        assert args.enable_agentic_auto_approve is False
        assert args.enable_agentic_dangerous_command_check is False
        
        # Test filter fields
        assert args.index_filter_level == 0
        assert args.index_filter_enable_relevance_verification is True
        assert args.index_filter_workers == 1
        assert args.index_filter_file_num == 10
        assert args.index_build_workers == 1
        
        # Test other model fields
        assert args.designer_model == ""
        assert args.file is None
        assert args.ray_address == ""
        assert args.anti_quota_limit == 1
        assert args.print_request is False
        assert args.py_packages == ""
        
        # Test search fields
        assert args.search == ""
        assert args.search_engine == ""
        assert args.search_engine_token == ""
        
        # Test RAG fields
        assert args.rag_url == ""
        assert args.rag_token == ""
        assert args.rag_type == "storage"
        assert args.rag_storage_type == "duckdb"
        assert args.rag_params_max_tokens == 500000
        assert args.rag_doc_filter_relevance == 2
        assert args.rag_context_window_limit == 120000
        assert args.rag_duckdb_vector_dim == 1024
        assert args.rag_duckdb_query_similarity == 0.1
        assert args.rag_duckdb_query_top_k == 10000
        assert args.rag_index_build_workers == 10
        assert args.rag_emb_dim == 1024
        assert args.rag_emb_text_size == 1024
        assert args.local_image_host == ""
        assert args.rag_recall_max_queries == 5
        assert args.rag_qa_conversation_strategy == "multi_round"
        
        # Test relevance and context fields
        assert args.verify_file_relevance_score == 6
        assert args.enable_rag_search is False
        assert args.enable_rag_context is False
        assert args.collection is None
        assert args.collections is None
        
        # Test merge and human fields
        assert args.auto_merge is False
        assert args.human_as_model is False
        assert args.human_model_num == 1
        
        # Test image fields
        assert args.image_file == ""
        assert args.image_mode == "direct"
        assert args.image_max_iter == 1
        
        # Test URL fields
        assert args.urls == ""
        assert args.urls_use_model is False
        assert args.command is None
        assert args.doc_command is None
        assert args.required_exts is None
        assert args.hybrid_index_max_output_tokens == 1000000
        
        # Test various enable fields
        assert args.enable_multi_round_generate is False
        assert args.monitor_mode is False
        assert args.enable_hybrid_index is False
        assert args.rag_build_name is None
        assert args.disable_auto_window is False
        assert args.filter_batch_size == 5
        assert args.disable_segment_reorder is False
        assert args.tokenizer_path is None
        assert args.skip_confirm is False
        assert args.silence is False
        assert args.exclude_files == ""
        assert args.output == ""
        assert args.single_file is False
        assert args.query_prefix is None
        assert args.query_suffix is None
        assert args.from_yaml is None
        assert args.base_dir is None
        assert args.context is None
        assert args.editblock_similarity == 0.9
        assert args.include_project_structure is False
        assert args.new_session is False
        
        # Test prompt and review fields
        assert args.prompt_review is None
        assert args.agent_designer_mode == "svg"
        
        # Test ratio fields
        assert args.full_text_ratio == 0.7
        assert args.segment_ratio == 0.2
        assert args.buff_ratio == 0.1
        
        # Test inference fields
        assert args.disable_inference_enhance is True
        assert args.inference_deep_thought is False
        assert args.inference_slow_without_deep_thought is False
        assert args.inference_compute_precision == 64
        assert args.without_contexts is False
        assert args.skip_events is False
        assert args.data_cells_max_num == 2000
        assert args.generate_times_same_model == 1
        assert args.rank_times_same_model == 1
        
        # Test strategy fields
        assert args.rank_strategy == "file"
        
        # Test action and mode fields
        assert args.action == []
        assert args.enable_global_memory is False
        assert args.product_mode == "lite"
        
        # Test reasoning content fields
        assert args.keep_reasoning_content is False
        assert args.keep_only_reasoning_content is False
        
        # Test code apply fields
        assert args.in_code_apply is False
        assert args.model_filter_path is None
        
        # Test conversation prune fields
        assert args.conversation_prune_safe_zone_tokens == 50*1024
        assert args.conversation_prune_group_size == 4
        assert args.conversation_prune_strategy == "summarize"
        
        # Test context prune fields
        assert args.context_prune_strategy == "extract"
        assert args.context_prune is True
        assert args.context_prune_safe_zone_tokens == 24*1024
        assert args.context_prune_sliding_window_size == 1000
        assert args.context_prune_sliding_window_overlap == 100
        
        # Test command fields
        assert args.auto_command_max_iterations == 10
        assert args.use_shell_commands is True
        assert args.skip_commit is False
        
        # Test beta and reproduction fields
        assert args.enable_beta is False
        assert args.how_to_reproduce is None
        
        # Test URL lists
        assert args.dynamic_urls == []
        assert args.add_updated_urls == []
        
        # Test history fields
        assert args.enable_task_history is False
        assert args.event_file is None
        
        # Test active context fields
        assert args.enable_active_context is False
        assert args.enable_active_context_in_generate is False
        
        # Test generate fields
        assert args.generate_max_rounds == 5
        
        # Test revert fields
        assert args.revert_commit_id is None
        
        # Test auto fix fields
        assert args.enable_auto_fix_lint is False
        assert args.enable_auto_fix_compile is False
        assert args.enable_auto_fix_merge is False
        assert args.auto_fix_lint_max_attempts == 5
        assert args.auto_fix_compile_max_attempts == 5
        assert args.auto_fix_merge_max_attempts == 5
        
        # Test rules fields
        assert args.enable_auto_select_rules is True
        
        # Test shadows fields
        assert args.ignore_clean_shadows is False 


"""
Utility functions for the Agent Hooks system.
"""

import os
from typing import Dict, Optional, Any
from ..agent_events.types import EventMessage
from .types import HookProcessingResult
from .hook_manager import HookManager
from .hook_executor import HookExecutor

async def process_event(event_message: EventMessage, 
                       config_path: Optional[str] = None,
                       cwd: Optional[str] = None,
                       command_timeout: int = 30000) -> HookProcessingResult:
    """
    Convenience function to process an event with default configuration.
    
    Args:
        event_message: The event to process
        config_path: Path to hooks configuration file
        cwd: Working directory for command execution
        command_timeout: Command timeout in milliseconds
        
    Returns:
        HookProcessingResult with execution results
    """
    manager = HookManager(
        config_path=config_path,
        cwd=cwd,
        command_timeout=command_timeout
    )
    return await manager.process_event(event_message)

def create_hook_executor(cwd: Optional[str] = None, 
                        timeout: int = 30000,
                        env: Optional[Dict[str, str]] = None) -> HookExecutor:
    """
    Create a hook executor with the specified configuration.
    
    Args:
        cwd: Working directory for command execution
        timeout: Command timeout in milliseconds
        env: Environment variables
        
    Returns:
        Configured HookExecutor instance
    """
    return HookExecutor(cwd=cwd, timeout=timeout, env=env)

def create_hook_manager(config_path: Optional[str] = None,
                       cwd: Optional[str] = None,
                       command_timeout: int = 30000) -> HookManager:
    """
    Create a hook manager with the specified configuration.
    
    Args:
        config_path: Path to hooks configuration file
        cwd: Working directory for command execution
        command_timeout: Command timeout in milliseconds
        
    Returns:
        Configured HookManager instance
    """
    return HookManager(
        config_path=config_path,
        cwd=cwd,
        command_timeout=command_timeout
    )

def config_exists(config_path: Optional[str] = None) -> bool:
    """
    Check if hooks configuration file exists.
    Supports auto-detection of JSON and YAML formats.
    
    Args:
        config_path: Path to hooks configuration file
        
    Returns:
        True if configuration file exists, False otherwise
    """
    if config_path is None:
        # Auto-detect configuration file
        base_dir = os.path.join(os.getcwd(), '.auto-coder')
        
        # Check for configuration files in priority order
        candidates = [
            os.path.join(base_dir, 'hooks.json'),
            os.path.join(base_dir, 'hooks.yaml'),
            os.path.join(base_dir, 'hooks.yml')
        ]
        
        for candidate in candidates:
            if os.path.exists(candidate):
                return True
        
        return False
    else:
        return os.path.exists(config_path) 


from typing import Generator, List, Dict, Union, Tuple, Optional
import os
import byzerllm
import pydantic
import git
from rich.console import Console
from autocoder.common.printer import Printer
from autocoder.common import AutoCoderArgs
from autocoder.common.utils_code_auto_generate import stream_chat_with_continue
from autocoder.common import SourceCode, SourceCodeList
from autocoder.common.action_yml_file_manager import ActionYmlFileManager


class AutoLearn:
    def __init__(self, llm: Union[byzerllm.ByzerLLM, byzerllm.SimpleByzerLLM],
                 args: AutoCoderArgs,
                 console: Optional[Console] = None):
        """
        初始化 AutoLearn

        Args:
            llm: ByzerLLM 实例，用于代码分析和学习
            args: AutoCoderArgs 实例，包含配置信息
            console: Rich Console 实例，用于输出
        """
        self.llm = llm
        self.args = args
        self.console = console or Console()
        self.printer = Printer()

    @byzerllm.prompt()
    def analyze_commit(self,
                       querie_with_urls_and_changes: List[Tuple[str, List[str], Dict[str, Tuple[str, str]]]],                       
                       new_query:str,            
                       ) -> str:
        """        
        下面是用户一次提交的代码变更：
        <changes>
        {% for query,urls,changes in querie_with_urls_and_changes %}
        ## 原始的任务需求
        {{ query }}

        修改的文件:
        {% for url in urls %}
        - {{ url }}
        {% endfor %}

        代码变更:
        {% for file_path, (before, after) in changes.items() %}
        ##File: {{ file_path }}
        ##修改前:

        {{ before or "New file" }}        

        ##File: {{ file_path }}
        ##修改后:

        {{ after or "File deleted" }}

        {% endfor %}
        {% endfor %}
        </changes>
        

        请对根据上面的代码变更进行深入分析，提取具有通用价值的功能模式和设计模式，转化为可在其他项目中复用的代码规则（rules）。
        
        - 识别代码变更中具有普遍应用价值的功能点和模式
        - 将这些功能点提炼为结构化规则，便于在其他项目中快速复用
        - 生成清晰的使用示例，包含完整依赖和调用方式
        - 这些规则将被存储在项目的 {{ rules_path }} 目录，供后续自动化代码生成使用   
        - 更新 {{rules_index_path}} 文件,确保改文件有新增文件的描述信息
        
        当前 {{rules_index_path}} 文件内容:
        {% if index_file_content %}                                
        {{ index_file_content }}      
        {% else %}
        <为空>
        {% endif %}
        - {{ new_query }} 
                                

        最后，新生成的文件格式要是这种形态的：
    
       <example_rules>
        ---
        description: [简明描述规则的功能，20字以内]
        globs: [匹配应用此规则的文件路径，如"src/services/*.py"]
        alwaysApply: [是否总是应用，通常为false]
        ---

        # [规则主标题]

        ## 简要说明
        [该规则的功能、适用场景和价值，100字以内]

        ## 典型用法
        ```python
        # 完整的代码示例，包含:
        # 1. 必要的import语句
        # 2. 类/函数定义
        # 3. 参数说明
        # 4. 调用方式
        # 5. 关键注释        
        ```

        ## 依赖说明
        - [必要的依赖库及版本]
        - [环境要求]
        - [初始化流程(如有)]

        ## 学习来源
        [从哪个提交变更的哪部分代码中提取的该功能点]    
        </example_rules>                
        """
        return {
            "project_root": os.path.abspath(self.args.source_dir),
            "index_file_content": self._get_index_file_content(),
            "rules_path": os.path.join(os.path.abspath(self.args.source_dir), ".autocoderrules"),
            "rules_index_path": os.path.join(os.path.abspath(self.args.source_dir), ".autocoderrules", "index.md")
        }

    @byzerllm.prompt()
    def analyze_modules(self, sources: SourceCodeList, query: str) -> str:
        """
        下面是用户提供的需要抽取规则的代码：
        <files>
        {% for source in sources.sources %}
        ##File: {{ source.module_name }}        
        {{ source.source_code }}        
        {% endfor %}
        </files>

        请对对上面的代码进行深入分析，提取具有通用价值的功能模式和设计模式，转化为可在其他项目中复用的代码规则（rules）。
        
        - 识别代码变更中具有普遍应用价值的功能点和模式
        - 将这些功能点提炼为结构化规则，便于在其他项目中快速复用
        - 生成清晰的使用示例，包含完整依赖和调用方式
        - 这些规则将被存储在项目的 {{ rules_path }} 目录，供后续自动化代码生成使用   
        - 更新 {{rules_index_path}} 文件,确保改文件有新增文件的描述信息
        
        当前 {{rules_index_path}} 文件内容:
        {% if index_file_content %}                                
        {{ index_file_content }}      
        {% else %}
        <为空>
        {% endif %}
        - {{ query }} 
                                
        最后，新生成的文件格式要是这种形态的：
    
        <example_rules>
        ---
        description: [简明描述规则的功能，20字以内]
        globs: [匹配应用此规则的文件路径，如"src/services/*.py"]
        alwaysApply: [是否总是应用，通常为false]
        ---

        # [规则主标题]

        ## 简要说明
        [该规则的功能、适用场景和价值，100字以内]

        ## 典型用法
        ```python
        # 完整的代码示例，包含:
        # 1. 必要的import语句
        # 2. 类/函数定义
        # 3. 参数说明
        # 4. 调用方式
        # 5. 关键注释        
        ```

        ## 依赖说明
        - [必要的依赖库及版本]
        - [环境要求]
        - [初始化流程(如有)]

        ## 学习来源
        [从哪个提交变更的哪部分代码中提取的该功能点]    
        </example_rules>   
        """

        # 获取索引文件内容
        index_file_content = self._get_index_file_content()

        return {
            "project_root": os.path.abspath(self.args.source_dir),
            "index_file_content": index_file_content,
            "rules_path": os.path.join(os.path.abspath(self.args.source_dir), ".autocoderrules"),
            "rules_index_path": os.path.join(os.path.abspath(self.args.source_dir), ".autocoderrules", "index.md")
        }

    def get_commit_changes(self, commit_id: str) -> Tuple[List[Tuple[str, List[str], Dict[str, Tuple[str, str]]]], Optional[str]]:
        """
        直接从Git仓库获取指定commit的变更

        Args:
            commit_id: Git commit的ID

        Returns:
            List[Tuple[str, List[str], Dict[str, Tuple[str, str]]]]: 包含查询、URL和变更信息的列表
        """
        printer = Printer()
        querie_with_urls_and_changes = []
        try:
            repo = git.Repo(self.args.source_dir)
            commit = repo.commit(commit_id)
            modified_files = []
            changes = {}

            # 检查是否是首次提交（没有父提交）
            if not commit.parents:
                # 首次提交，获取所有文件
                for item in commit.tree.traverse():
                    if item.type == 'blob':  # 只处理文件，不处理目录
                        file_path = item.path
                        modified_files.append(file_path)
                        # 首次提交前没有内容
                        before_content = None
                        # 获取提交后的内容
                        after_content = repo.git.show(
                            f"{commit.hexsha}:{file_path}")
                        changes[file_path] = (before_content, after_content)
            else:
                # 获取parent commit
                parent = commit.parents[0]
                # 获取变更的文件列表
                for diff_item in parent.diff(commit):
                    file_path = diff_item.a_path if diff_item.a_path else diff_item.b_path
                    modified_files.append(file_path)

                    # 获取变更前内容
                    before_content = None
                    try:
                        if diff_item.a_blob:
                            before_content = repo.git.show(
                                f"{parent.hexsha}:{file_path}")
                    except git.exc.GitCommandError:
                        pass  # 文件可能是新增的

                    # 获取变更后内容
                    after_content = None
                    try:
                        if diff_item.b_blob:
                            after_content = repo.git.show(
                                f"{commit.hexsha}:{file_path}")
                    except git.exc.GitCommandError:
                        pass  # 文件可能被删除

                    changes[file_path] = (before_content, after_content)

            # 使用commit消息作为查询内容
            query = commit.message
            querie_with_urls_and_changes.append(
                (query, modified_files, changes))

        except git.exc.GitCommandError as e:
            printer.print_in_terminal(
                "git_command_error", style="red", error=str(e))
        except Exception as e:
            printer.print_in_terminal(
                "get_commit_changes_error", style="red", error=str(e))

        return querie_with_urls_and_changes, None

    def analyze_commit_changes(self, query: str, commit_id: str, conversations: List[Dict] = []) -> Optional[Generator[str, None, None]]:
        """
        分析指定commit的代码变更

        Args:
            query: 用户的查询/要求
            commit_id: 指定的commit ID
            conversations: 之前的对话历史 (可选)

        Returns:
            Optional[Generator]: 分析结果生成器，如果出错则返回None
        """
        printer = Printer()

        # 获取commit的变更信息
        changes, _ = self.get_commit_changes(commit_id)

        if not changes:
            printer.print_in_terminal("no_commit_changes", style="red")
            return None

        # 调用LLM进行代码分析
        try:
            # 获取prompt内容
            prompt_content = self.analyze_commit.prompt(
                querie_with_urls_and_changes=changes,
                new_query=query
            )

            # 准备对话历史
            if conversations:
                new_conversations = conversations[:-1]
            else:
                new_conversations = []
            new_conversations.append(
                {"role": "user", "content": prompt_content})

            # 调用LLM
            v = stream_chat_with_continue(
                llm=self.llm,
                conversations=new_conversations,
                llm_config={},
                args=self.args
            )
            return v
        except Exception as e:
            printer.print_in_terminal(
                "commit_analysis_error", style="red", error=str(e))
            return None

    def _get_index_file_content(self) -> str:
        """获取索引文件内容"""
        index_file_path = os.path.join(os.path.abspath(
            self.args.source_dir), ".autocoderrules", "index.md")
        index_file_content = ""

        try:
            if os.path.exists(index_file_path):
                with open(index_file_path, 'r', encoding='utf-8') as f:
                    index_file_content = f.read()
        except Exception as e:
            self.printer.print_str_in_terminal(
                f"读取索引文件时出错: {str(e)}", style="yellow")

        return index_file_content

    def analyze(self, sources: SourceCodeList, query: str, conversations: List[Dict] = []) -> Optional[Generator[str, None, None]]:
        """
        分析给定的模块文件，根据用户需求生成可复用功能点的总结。

        Args:
            sources: 包含模块路径和内容的 SourceCodeList 对象。
            query: 用户的具体分析要求。
            conversations: 之前的对话历史 (可选)。

        Returns:
            Optional[Generator]: LLM 返回的分析结果生成器，如果出错则返回 None。
        """
        if not sources or not sources.sources:
            self.printer.print_str_in_terminal("没有提供有效的模块文件进行分析。", style="red")
            return None

        try:
            # 准备 Prompt
            prompt_content = self.analyze_modules.prompt(
                sources=sources,
                query=query
            )

            # 准备对话历史
            # 如果提供了 conversations，我们假设最后一个是用户的原始查询，替换它
            if conversations:
                new_conversations = conversations[:-1]
            else:
                new_conversations = []
            new_conversations.append(
                {"role": "user", "content": prompt_content})

            # 调用 LLM
            v = stream_chat_with_continue(
                llm=self.llm,
                conversations=new_conversations,
                llm_config={},
                args=self.args
            )
            return v
        except Exception as e:
            self.printer.print_in_terminal(
                "代码分析时出错", style="red", error=str(e))
            return None



"""
TodoReadTool 模块

该模块实现了 TodoReadTool 和 TodoReadToolResolver 类，用于在 BaseAgent 框架中
提供 TODO 列表读取功能。
"""

from typing import Dict, Any, List, Optional

import byzerllm
from loguru import logger

from autocoder.agent.base_agentic.types import BaseTool, ToolResult
from autocoder.agent.base_agentic.tool_registry import ToolRegistry
from autocoder.agent.base_agentic.tools.base_tool_resolver import BaseToolResolver
from autocoder.agent.base_agentic.types import ToolDescription, ToolExample
from autocoder.common.todos.get_todo_manager import get_todo_manager
from autocoder.common.todos.exceptions import TodoManagerError


class TodoReadTool(BaseTool):
    """读取TODO列表工具"""
    pass  # 不需要参数


class TodoReadToolResolver(BaseToolResolver):
    """TODO读取工具解析器，实现读取逻辑"""
    def __init__(self, agent, tool, args):
        super().__init__(agent, tool, args)
        self.tool: TodoReadTool = tool
        
        # 初始化 todo 管理器
        self.todo_manager = get_todo_manager()
    
    def _format_todo_display(self, todos: List[Dict[str, Any]]) -> str:
        """格式化 TODO 显示"""
        if not todos:
            return "当前会话没有找到 TODO 项。"
        
        output = []
        output.append("=== 当前会话 TODO 列表 ===\n")
        
        # 状态表情符号映射
        status_emoji = {
            'pending': '⏳',
            'in_progress': '🔄',
            'completed': '✅',
            'cancelled': '❌'
        }
        
        # 优先级图标映射
        priority_icon = {
            'high': '🔴',
            'medium': '🟡',
            'low': '🟢'
        }
        
        # 显示带状态标记的单个列表中的 todos
        for idx, todo in enumerate(todos, 1):
            status = todo.get('status', 'pending')
            priority = todo.get('priority', 'medium')
            todo_id = todo.get('todo_id', 'MISSING_ID')
            todo_content = todo.get('content', '缺少内容')
            
            # 记录任何缺少 ID 的 todos
            if todo_id == 'MISSING_ID':
                logger.error(f"发现缺少 ID 的 TODO: {todo}")
            
            # 获取状态表情符号和优先级图标
            status_mark = status_emoji.get(status, '❓')
            priority_mark = priority_icon.get(priority, '⚪')
            
            # 格式化每个 todo 项
            output.append(f"{idx}. {status_mark} {priority_mark} [{todo_id}] {todo_content}")
            
            # 如果存在，添加备注
            if todo.get('notes'):
                output.append(f"   └─ 📝 {todo['notes']}")
            
            # 如果存在，添加依赖
            if todo.get('dependencies'):
                deps = ', '.join(todo['dependencies'])
                output.append(f"   └─ 🔗 依赖于: {deps}")
        
        output.append("")
        
        # 使用 todo 管理器添加摘要统计
        try:
            # 获取当前会话 ID
            conversation_id = None
            if hasattr(self.agent, 'conversation_config') and self.agent.conversation_config:
                conversation_id = self.agent.conversation_config.conversation_id
            
            stats = self.todo_manager.get_statistics(conversation_id=conversation_id)
            total = stats.get('total', 0)
            status_counts = {
                'pending': stats.get('pending', 0),
                'in_progress': stats.get('in_progress', 0),
                'completed': stats.get('completed', 0),
                'cancelled': stats.get('cancelled', 0)
            }
            
            summary_parts = [f"总计: {total}"]
            for status, emoji in status_emoji.items():
                if status in status_counts and status_counts[status] > 0:
                    status_label = {
                        'pending': '待处理',
                        'in_progress': '进行中',
                        'completed': '已完成',
                        'cancelled': '已取消'
                    }.get(status, status)
                    summary_parts.append(f"{emoji} {status_label}: {status_counts[status]}")
            
            output.append("📊 摘要: " + " | ".join(summary_parts))
        except Exception as e:
            logger.error(f"获取统计信息失败: {e}")
            output.append("📊 摘要: 统计信息不可用")
        
        return "\n".join(output)

    def resolve(self) -> ToolResult:
        """实现 TODO 读取工具的解析逻辑"""
        try:
            logger.info("读取当前 TODO 列表")
            
            # 从 todo 管理器加载 todos
            conversation_id = None
            if hasattr(self.agent, 'conversation_config') and self.agent.conversation_config:
                conversation_id = self.agent.conversation_config.conversation_id
            
            todos = self.todo_manager.get_todos(conversation_id=conversation_id)
            
            # 格式化显示
            formatted_display = self._format_todo_display(todos)
            
            logger.info(f"在当前会话中找到 {len(todos)} 个 TODO 项")
            
            return ToolResult(
                success=True,
                message="成功检索 TODO 列表。",
                content=formatted_display
            )
            
        except TodoManagerError as e:
            logger.error(f"TODO 管理器读取 TODO 列表时出错: {e}")
            return ToolResult(
                success=False,
                message=f"读取 TODO 列表失败: {str(e)}",
                content=None
            )
        except Exception as e:
            logger.error(f"读取 TODO 列表时出错: {e}")
            import traceback
            return ToolResult(
                success=False,
                message=f"读取 TODO 列表失败: {str(e)}",
                content=traceback.format_exc()
            )


class TodoReadToolDescGenerator:
    def __init__(self, params: Dict[str, Any]):
        self.params = params

    @byzerllm.prompt()
    def todo_read_description(self) -> Dict:
        """
        Description: Request to read and display the current session's TODO list. Use this tool when you need to check the current status of tasks, track progress, or see what work remains to be completed in the current conversation session.
        Parameters:
        No parameters required
        Usage:
        <todo_read>
        </todo_read>
        """
        return self.params


def register_todo_read_tool():
    """Register TODO read tool"""
    desc_gen = TodoReadToolDescGenerator({})
    
    # 准备工具描述
    description = ToolDescription(
        description=desc_gen.todo_read_description.prompt()
    )
    
    # 准备工具示例
    example = ToolExample(
        title="TODO read tool usage example",
        body="""<todo_read>
</todo_read>"""
    )
    
    # 注册工具
    ToolRegistry.register_tool(
        tool_tag="todo_read",  # XML标签名
        tool_cls=TodoReadTool,  # 工具类
        resolver_cls=TodoReadToolResolver,  # 解析器类
        description=description,  # 工具描述
        example=example,  # 工具示例
        use_guideline="Use this tool to read and display the current session's TODO list, helping you track task progress and understand outstanding work. Ideal for checking task status and understanding project progress during complex multi-step operations."  # 使用指南
    )




"""
Markdown 处理器模块

提供智能 Markdown 解析和分割功能，支持多种分割模式。
"""

import re
import yaml
import hashlib
from enum import Enum
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional, Callable
from pathlib import Path
from autocoder.common.international import get_message, get_message_with_format


class SplitMode(Enum):
    """分割模式枚举"""
    DELIMITER = "delimiter"          # 按分隔符分割（兼容模式）
    HEADING1 = "h1"                 # 按 H1 标题分割
    HEADING2 = "h2"                 # 按 H2 标题分割
    HEADING3 = "h3"                 # 按 H3 标题分割
    ANY_HEADING = "any"             # 按任何标题分割
    FRONT_MATTER = "frontmatter"    # 按 YAML front matter 分割
    CUSTOM_PATTERN = "custom"       # 按自定义正则模式分割


@dataclass
class SplitterConfig:
    """分割器配置"""
    pattern: str = ""                                    # 自定义正则模式
    min_length: int = 50                                # 最小文档长度
    max_length: int = 10000                             # 最大文档长度
    overlap_size: int = 100                             # 重叠大小（用于长文档分割）
    preserve_context: bool = True                       # 是否保留上下文
    custom_splitter: Optional[Callable[[str], List[str]]] = None  # 自定义分割函数


@dataclass
class DocumentPart:
    """文档部分"""
    content: str
    title: str
    level: int = 1
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class Document:
    """Markdown 文档"""
    original_file: str
    index: int
    content: str
    temp_filename: str
    title: str
    heading_level: int = 1
    metadata: Dict[str, Any] = field(default_factory=dict)


class MarkdownProcessor:
    """Markdown 处理器，用于解析和分割 Markdown 内容"""
    
    def __init__(self):
        """初始化处理器"""
        self.split_mode = SplitMode.HEADING1
        self.delimiter = "==="
        self.min_heading_level = 1
        self.max_heading_level = 6
        self.config = SplitterConfig()
    
    def process_content(self, content: str, filename: str = "stdin") -> List[Document]:
        """
        处理 Markdown 内容，返回分割后的文档列表
        
        Args:
            content: Markdown 内容
            filename: 文件名
            
        Returns:
            分割后的文档列表
        """
        parts = []
        
        # 首先检测文档是否包含多个独立文档
        if self._contains_multiple_documents(content):
            print(get_message("detected_multi_document_structure"))
            try:
                parts = self._parse_multiple_documents(content)
            except Exception as e:
                print(get_message_with_format("multi_document_parse_failed_fallback", error=str(e)))
                parts = []
        
        # 如果没有检测到多文档或解析失败，使用原有分割逻辑
        if not parts:
            if self.split_mode != SplitMode.DELIMITER:
                try:
                    parts = self._split_by_headings(content)
                    if len(parts) <= 1:
                        # 如果没有找到标题，回退到分隔符模式
                        parts = self._split_by_delimiter(content)
                except Exception:
                    # 如果解析失败，回退到分隔符模式
                    parts = self._split_by_delimiter(content)
            else:
                # 使用分隔符模式
                parts = self._split_by_delimiter(content)
        
        # 使用自定义分割器（如果配置了）
        if self.config.custom_splitter:
            print(get_message("using_custom_splitter"))
            custom_parts = self.config.custom_splitter(content)
            parts = self._convert_to_document_parts(custom_parts)
        
        # 如果没有找到任何分割，将整个内容作为一个文档
        if not parts:
            parts = [DocumentPart(
                content=content.strip(),
                title=self._extract_title(content),
                level=1
            )]
        
        # 应用长度和重叠配置
        parts = self._apply_length_constraints(parts)
        
        # 转换为 Document 对象
        documents = []
        base_name = self._get_base_name(filename)
        
        for i, part in enumerate(parts):
            doc = Document(
                original_file=filename,
                index=i,
                content=part.content,
                title=part.title,
                heading_level=part.level,
                temp_filename=self._generate_temp_filename(base_name, i, len(parts), part.content),
                metadata=part.metadata
            )
            documents.append(doc)
        
        return documents
    
    def _contains_multiple_documents(self, content: str) -> bool:
        """检测内容是否包含多个独立文档"""
        # 检测 YAML front matter 分隔符
        front_matter_pattern = re.compile(r'^---\s*$', re.MULTILINE)
        matches = front_matter_pattern.findall(content)
        if len(matches) >= 3:  # 至少有3个分隔符才认为是多文档
            return True
        
        # 检测连续的空行分隔（可能是多文档）
        empty_line_pattern = re.compile(r'\n\s*\n\s*\n')
        if len(empty_line_pattern.findall(content)) >= 2:
            # 进一步检查是否有标题结构
            heading_pattern = re.compile(r'^#{1,6}\s+.+$', re.MULTILINE)
            headings = heading_pattern.findall(content)
            return len(headings) >= 2
        
        return False
    
    def _parse_multiple_documents(self, content: str) -> List[DocumentPart]:
        """解析多文档结构"""
        parts = []
        
        # 尝试按 YAML front matter 分割
        if "---" in content:
            yaml_parts = self._split_by_front_matter(content)
            if len(yaml_parts) > 1:
                parts = yaml_parts
        
        # 如果没有找到 YAML 分割，抛出异常
        if not parts:
            raise ValueError(get_message("cannot_parse_multi_document_structure"))
        
        return parts
    
    def _split_by_front_matter(self, content: str) -> List[DocumentPart]:
        """按 YAML front matter 分割"""
        doc_parts = []
        lines = content.split('\n')
        current_doc = []
        current_yaml = []
        in_yaml = False
        in_content = False
        
        for line in lines:
            trimmed = line.strip()
            
            if trimmed == "---":
                if not in_yaml and not in_content:
                    # 开始新的 YAML block
                    in_yaml = True
                    continue
                elif in_yaml and not in_content:
                    # YAML block 结束，内容开始
                    in_yaml = False
                    in_content = True
                    continue
                elif in_content:
                    # 当前文档结束，保存并开始新文档
                    if current_doc or current_yaml:
                        self._add_front_matter_doc(doc_parts, '\n'.join(current_yaml), '\n'.join(current_doc))
                    
                    # 重置状态
                    current_doc = []
                    current_yaml = []
                    in_yaml = True
                    in_content = False
                    continue
            
            if in_yaml:
                current_yaml.append(line)
            elif in_content:
                current_doc.append(line)
            else:
                # 没有 YAML header 的情况，直接当作内容处理
                current_doc.append(line)
                in_content = True
        
        # 添加最后一个文档
        if current_doc or current_yaml:
            self._add_front_matter_doc(doc_parts, '\n'.join(current_yaml), '\n'.join(current_doc))
        
        # 如果没有找到任何文档，返回整个内容作为一个文档
        if not doc_parts:
            doc_parts = [DocumentPart(
                content=content.strip(),
                title=self._extract_title(content),
                level=1,
                metadata={}
            )]
        
        return doc_parts
    
    def _add_front_matter_doc(self, doc_parts: List[DocumentPart], yaml_content: str, markdown_content: str):
        """添加一个 front matter 文档"""
        metadata = self._parse_yaml_metadata(yaml_content)
        content = markdown_content.strip()
        
        title = self._extract_title(content)
        if 'title' in metadata and metadata['title']:
            title = str(metadata['title'])
        
        if content or metadata:
            doc_parts.append(DocumentPart(
                content=content,
                title=title,
                level=1,
                metadata=metadata
            ))
    
    def _parse_yaml_metadata(self, yaml_content: str) -> Dict[str, Any]:
        """解析 YAML 元数据"""
        if not yaml_content.strip():
            return {}
        
        try:
            return yaml.safe_load(yaml_content) or {}
        except yaml.YAMLError:
            # 简单解析失败时的备用方案
            metadata = {}
            lines = yaml_content.split('\n')
            
            for line in lines:
                line = line.strip()
                if not line or line.startswith('#'):
                    continue
                
                if ':' in line:
                    parts = line.split(':', 1)
                    if len(parts) == 2:
                        key = parts[0].strip()
                        value = parts[1].strip().strip('"\'')
                        metadata[key] = value
            
            return metadata
    
    def _convert_to_document_parts(self, parts: List[str]) -> List[DocumentPart]:
        """将字符串数组转换为 DocumentPart"""
        doc_parts = []
        for part in parts:
            if part.strip():
                doc_parts.append(DocumentPart(
                    content=part.strip(),
                    title=self._extract_title(part),
                    level=1
                ))
        return doc_parts
    
    def _apply_length_constraints(self, parts: List[DocumentPart]) -> List[DocumentPart]:
        """应用长度约束和重叠配置"""
        result = []
        too_short_parts = []
        
        for part in parts:
            # 如果文档太短，先保存起来
            if len(part.content) < self.config.min_length:
                too_short_parts.append(part)
                continue
            
            # 如果文档太长，分割
            if len(part.content) > self.config.max_length:
                sub_parts = self._split_long_document(part)
                result.extend(sub_parts)
            else:
                result.append(part)
        
        # 如果所有文档都被过滤掉了（因为太短），至少保留一个最长的
        if not result and too_short_parts:
            longest = max(too_short_parts, key=lambda x: len(x.content))
            result.append(longest)
        
        return result
    
    def _split_long_document(self, part: DocumentPart) -> List[DocumentPart]:
        """分割过长的文档"""
        parts = []
        content = part.content
        max_len = self.config.max_length
        overlap = self.config.overlap_size
        
        # 安全检查：防止无限循环
        max_iterations = 100
        iterations = 0
        
        while len(content) > max_len and iterations < max_iterations:
            iterations += 1
            
            # 找到合适的分割点
            split_pos = self._find_best_split_position(content, max_len)
            
            # 安全检查
            if split_pos <= 0:
                split_pos = max_len // 2
            if split_pos >= len(content):
                break
            
            # 创建当前部分
            current_part = DocumentPart(
                content=content[:split_pos],
                title=f"{part.title} (第{len(parts)+1}部分)",
                level=part.level,
                metadata=part.metadata.copy()
            )
            parts.append(current_part)
            
            # 计算下一部分的起始位置（考虑重叠）
            next_start = max(0, split_pos - overlap)
            if next_start >= split_pos:
                next_start = split_pos
            
            if next_start >= len(content):
                break
            
            content = content[next_start:]
            
            if len(content) <= overlap:
                break
        
        # 添加最后一部分
        if len(content) > self.config.min_length:
            parts.append(DocumentPart(
                content=content,
                title=f"{part.title} (第{len(parts)+1}部分)",
                level=part.level,
                metadata=part.metadata.copy()
            ))
        
        # 如果没有成功分割，返回原文档
        if not parts:
            return [part]
        
        return parts
    
    def _find_best_split_position(self, content: str, max_pos: int) -> int:
        """找到最佳分割位置"""
        if max_pos >= len(content):
            return len(content)
        
        # 优先在句号后分割
        for i in range(max_pos - 1, max(0, max_pos - 100), -1):
            if i < len(content) and content[i] == '.' and i + 1 < len(content) and content[i + 1] == ' ':
                return i + 1
        
        # 其次在换行符处分割
        for i in range(max_pos - 1, max(0, max_pos - 100), -1):
            if i < len(content) and content[i] == '\n':
                return i + 1
        
        # 最后在空格处分割
        for i in range(max_pos - 1, max(0, max_pos - 50), -1):
            if i < len(content) and content[i] == ' ':
                return i + 1
        
        return max_pos
    
    def _split_by_headings(self, content: str) -> List[DocumentPart]:
        """按标题分割 Markdown 内容"""
        lines = content.split('\n')
        parts = []
        current_part = DocumentPart(content="", title="", level=1)
        current_lines = []
        
        for line in lines:
            # 检查是否是标题行
            if line.strip().startswith('#'):
                level = self._get_heading_level(line)
                title = self._extract_title_from_line(line)
                
                # 如果这是我们要分割的标题级别
                if self._should_split_at_level(level):
                    # 保存之前的部分
                    if current_lines:
                        current_part.content = '\n'.join(current_lines)
                        if not current_part.title:
                            current_part.title = self._extract_title(current_part.content)
                            current_part.level = 1
                        parts.append(current_part)
                    
                    # 开始新的部分
                    current_part = DocumentPart(
                        content="",
                        title=title,
                        level=level
                    )
                    current_lines = [line]
                else:
                    # 不是分割级别的标题，添加到当前部分
                    current_lines.append(line)
            else:
                # 普通行，添加到当前部分
                current_lines.append(line)
        
        # 添加最后一个部分
        if current_lines:
            current_part.content = '\n'.join(current_lines)
            if not current_part.title:
                current_part.title = self._extract_title(current_part.content)
                current_part.level = 1
            parts.append(current_part)
        
        # 如果没有找到任何部分，使用整个内容
        if not parts:
            parts = [DocumentPart(
                content=content,
                title=self._extract_title(content),
                level=1
            )]
        
        return parts
    
    def _get_heading_level(self, line: str) -> int:
        """获取标题级别"""
        trimmed = line.strip()
        level = 0
        for char in trimmed:
            if char == '#':
                level += 1
            else:
                break
        return level
    
    def _extract_title_from_line(self, line: str) -> str:
        """从标题行提取标题文本"""
        title = re.sub(r'^#+\s*', '', line.strip())
        title = title.strip()
        return title if title else "Untitled"
    
    def _should_split_at_level(self, level: int) -> bool:
        """判断是否应该在指定级别分割"""
        if self.split_mode == SplitMode.HEADING1:
            return level == 1
        elif self.split_mode == SplitMode.HEADING2:
            return level <= 2
        elif self.split_mode == SplitMode.HEADING3:
            return level <= 3
        elif self.split_mode == SplitMode.ANY_HEADING:
            return self.min_heading_level <= level <= self.max_heading_level
        else:
            return False
    
    def _split_by_delimiter(self, content: str) -> List[DocumentPart]:
        """按分隔符分割内容（向后兼容）"""
        parts = content.split(self.delimiter)
        result = []
        
        for part in parts:
            trimmed = part.strip()
            if trimmed:
                result.append(DocumentPart(
                    content=trimmed,
                    title=self._extract_title(trimmed),
                    level=1
                ))
        
        return result
    
    def _extract_title(self, content: str) -> str:
        """从内容中提取标题"""
        lines = content.split('\n')
        for line in lines:
            line = line.strip()
            if line.startswith('#'):
                # 移除 # 符号和前后空格
                title = re.sub(r'^#+\s*', '', line)
                title = title.strip()
                if title:
                    return title
        
        # 如果没有找到标题，使用前50个字符
        if len(content) > 50:
            return content[:50] + "..."
        return content
    
    def _get_base_name(self, filename: str) -> str:
        """获取文件的基本名称（不包含扩展名）"""
        if not filename or filename == "stdin":
            return "stdin"
        
        return Path(filename).stem
    
    def _generate_temp_filename(self, base_name: str, index: int, total: int, content: str) -> str:
        """生成临时文件名，基于内容MD5哈希"""
        # 生成内容的MD5哈希（取前8位）
        content_hash = hashlib.md5(content.encode('utf-8')).hexdigest()[:8]
        
        if total == 1:
            return f"{base_name}_{content_hash}.md"
        
        return f"{base_name}_{index+1:02d}_{content_hash}.md"
    
    def _sanitize_filename(self, title: str) -> str:
        """清理文件名，移除不安全字符"""
        # 移除或替换不安全的文件名字符
        safe = re.sub(r'[<>:"/\\|?*\s]+', '_', title)
        
        # 移除多余的下划线
        safe = re.sub(r'_+', '_', safe)
        
        # 移除开头和结尾的下划线
        safe = safe.strip('_')
        
        # 限制长度
        if len(safe) > 50:
            safe = safe[:50]
        
        return safe.lower()
    
    def validate_content(self, content: str) -> None:
        """验证 Markdown 内容"""
        if not content.strip():
            raise ValueError(get_message("content_empty"))
    
    def get_document_info(self, doc: Document) -> str:
        """获取文档信息摘要"""
        content_preview = doc.content
        if len(content_preview) > 100:
            content_preview = content_preview[:100] + "..."
        
        title = doc.title if doc.title else "无标题"
        
        return (f"标题: {title} (H{doc.heading_level}), 文件: {doc.original_file}, "
                f"部分: {doc.index+1}, 临时文件: {doc.temp_filename}, 内容预览: {content_preview}")
    
    def set_split_mode(self, mode: SplitMode):
        """设置分割模式"""
        self.split_mode = mode
    
    def set_delimiter(self, delimiter: str):
        """设置自定义分隔符（用于兼容模式）"""
        if delimiter:
            self.delimiter = delimiter
        self.split_mode = SplitMode.DELIMITER
    
    def set_heading_level_range(self, min_level: int, max_level: int):
        """设置标题级别范围"""
        if 1 <= min_level <= 6 and min_level <= max_level <= 6:
            self.min_heading_level = min_level
            self.max_heading_level = max_level
    
    def set_custom_splitter(self, splitter: Callable[[str], List[str]]):
        """设置自定义分割器"""
        self.config.custom_splitter = splitter
    
    def set_length_constraints(self, min_length: int, max_length: int):
        """设置长度约束"""
        self.config.min_length = min_length
        self.config.max_length = max_length
    
    def set_overlap_size(self, size: int):
        """设置重叠大小"""
        self.config.overlap_size = size




import os
from byzerllm.utils.client import code_utils
from autocoder.common import AutoCoderArgs, git_utils
from autocoder.common.action_yml_file_manager import ActionYmlFileManager
from autocoder.common.text_similarity import TextSimilarity
from autocoder.memory.active_context_manager import ActiveContextManager
from autocoder.common.core_config import get_memory_manager
import pydantic
import byzerllm

import hashlib
import subprocess
import tempfile
from rich.console import Console
from rich.panel import Panel
from rich.syntax import Syntax
import json
from typing import Union, List, Tuple
from autocoder.common.types import CodeGenerateResult, MergeCodeWithoutEffect
from autocoder.common.code_modification_ranker import CodeModificationRanker
from autocoder.common import files as FileUtils
from autocoder.common.printer import Printer

class PathAndCode(pydantic.BaseModel):
    path: str
    content: str


class CodeAutoMergeEditBlock:
    def __init__(
        self,
        llm: byzerllm.ByzerLLM,
        args: AutoCoderArgs,
        fence_0: str = "```",
        fence_1: str = "```",
    ):
        self.llm = llm
        self.args = args
        self.fence_0 = fence_0
        self.fence_1 = fence_1
        self.printer = Printer()
    
    def parse_whole_text(self, text: str) -> List[PathAndCode]:
        '''
        从文本中抽取如下格式代码(two_line_mode)：

        ```python
        ##File: /project/path/src/autocoder/index/index.py
        <<<<<<< SEARCH
        =======
        >>>>>>> REPLACE
        ```

        或者 (one_line_mode)

        ```python:/project/path/src/autocoder/index/index.py
        <<<<<<< SEARCH
        =======
        >>>>>>> REPLACE
        ```

        '''
        HEAD = "<<<<<<< SEARCH"
        DIVIDER = "======="
        UPDATED = ">>>>>>> REPLACE"
        lines = text.split("\n")
        lines_len = len(lines)
        start_marker_count = 0
        block = []
        path_and_code_list = []
        # two_line_mode or one_line_mode
        current_editblock_mode = "two_line_mode"
        current_editblock_path = None

        def guard(index):
            return index + 1 < lines_len

        def start_marker(line, index):
            nonlocal current_editblock_mode
            nonlocal current_editblock_path
            if (
                line.startswith(self.fence_0)
                and guard(index)
                and ":" in line
                and lines[index + 1].startswith(HEAD)
            ):

                current_editblock_mode = "one_line_mode"
                current_editblock_path = line.split(":", 1)[1].strip()
                return True

            if (
                line.startswith(self.fence_0)
                and guard(index)
                and lines[index + 1].startswith("##File:")
            ):
                current_editblock_mode = "two_line_mode"
                current_editblock_path = None
                return True

            return False

        def end_marker(line, index):
            return line.startswith(self.fence_1) and UPDATED in lines[index - 1]

        for index, line in enumerate(lines):
            if start_marker(line, index) and start_marker_count == 0:
                start_marker_count += 1
            elif end_marker(line, index) and start_marker_count == 1:
                start_marker_count -= 1
                if block:
                    if current_editblock_mode == "two_line_mode":
                        path = block[0].split(":", 1)[1].strip()
                        content = "\n".join(block[1:])
                    else:
                        path = current_editblock_path
                        content = "\n".join(block)
                    block = []
                    path_and_code_list.append(
                        PathAndCode(path=path, content=content))
            elif start_marker_count > 0:
                block.append(line)

        return path_and_code_list

    def merge_code(self, generate_result: CodeGenerateResult, force_skip_git: bool = False):
        result = self.choose_best_choice(generate_result)
        self._merge_code(result.contents[0], force_skip_git)
        return result

    def choose_best_choice(self, generate_result: CodeGenerateResult) -> CodeGenerateResult:
        if len(generate_result.contents) == 1:
            return generate_result
        
        merge_results = []
        for content,conversations in zip(generate_result.contents,generate_result.conversations):
            merge_result = self._merge_code_without_effect(content)
            merge_results.append(merge_result)

        # If all merge results are None, return first one
        if all(len(result.failed_blocks) != 0 for result in merge_results):
            self.printer.print_in_terminal("all_merge_results_failed")
            return CodeGenerateResult(contents=[generate_result.contents[0]], conversations=[generate_result.conversations[0]])
        
        # If only one merge result is not None, return that one
        not_none_indices = [i for i, result in enumerate(merge_results) if len(result.failed_blocks) == 0]
        if len(not_none_indices) == 1:
            idx = not_none_indices[0]
            self.printer.print_in_terminal("only_one_merge_result_success")
            return CodeGenerateResult(contents=[generate_result.contents[idx]], conversations=[generate_result.conversations[idx]])        

        # 最后，如果有多个，那么根据质量排序再返回
        ranker = CodeModificationRanker(self.llm, self.args)
        ranked_result = ranker.rank_modifications(generate_result,merge_results)        
         
        ## 得到的结果，再做一次合并，第一个通过的返回 , 返回做合并有点重复低效，未来修改。
        for content,conversations in zip(ranked_result.contents,ranked_result.conversations):
            merge_result = self._merge_code_without_effect(content)
            if not merge_result.failed_blocks:
                return CodeGenerateResult(contents=[content], conversations=[conversations])

        # 最后保底，但实际不会出现
        return CodeGenerateResult(contents=[ranked_result.contents[0]], conversations=[ranked_result.conversations[0]])

    @byzerllm.prompt()
    def git_require_msg(self, source_dir: str, error: str) -> str:
        """
        auto_merge only works for git repositories.

        Try to use git init in the source directory.

        ```shell
        cd {{ source_dir }}
        git init .
        ```

        Then try to run auto-coder again.
        Error: {{ error }}
        """

    def get_edits(self, content: str):
        edits = self.parse_whole_text(content)
        HEAD = "<<<<<<< SEARCH"
        DIVIDER = "======="
        UPDATED = ">>>>>>> REPLACE"
        result = []
        for edit in edits:
            heads = []
            updates = []
            c = edit.content
            in_head = False
            in_updated = False
            for line in c.splitlines():
                if line.strip() == HEAD:
                    in_head = True
                    continue
                if line.strip() == DIVIDER:
                    in_head = False
                    in_updated = True
                    continue
                if line.strip() == UPDATED:
                    in_head = False
                    in_updated = False
                    continue
                if in_head:
                    heads.append(line)
                if in_updated:
                    updates.append(line)
            result.append((edit.path, "\n".join(heads), "\n".join(updates)))
        return result        

    def _merge_code_without_effect(self, content: str) -> MergeCodeWithoutEffect:
        """Merge code without any side effects like git operations, linting or file writing.
        Returns a tuple of:
        - list of (file_path, new_content) tuples for successfully merged blocks
        - list of (file_path, head, update) tuples for failed to merge blocks"""
        codes = self.get_edits(content)
        file_content_mapping = {}
        failed_blocks = []

        for block in codes:
            file_path, head, update = block
            if not os.path.exists(file_path):
                file_content_mapping[file_path] = update
            else:
                if file_path not in file_content_mapping:
                    file_content_mapping[file_path] = FileUtils.read_file(file_path)
                existing_content = file_content_mapping[file_path]

                # First try exact match
                new_content = (
                    existing_content.replace(head, update, 1)
                    if head
                    else existing_content + "\n" + update
                )

                # If exact match fails, try similarity match
                if new_content == existing_content and head:
                    similarity, best_window = TextSimilarity(
                        head, existing_content
                    ).get_best_matching_window()
                    if similarity > self.args.editblock_similarity:
                        new_content = existing_content.replace(
                            best_window, update, 1
                        )

                if new_content != existing_content:
                    file_content_mapping[file_path] = new_content
                else:
                    failed_blocks.append((file_path, head, update))

        return MergeCodeWithoutEffect(
            success_blocks=[(path, content)
                            for path, content in file_content_mapping.items()],
            failed_blocks=failed_blocks
        )
    

    def _merge_code(self, content: str, force_skip_git: bool = False):
        file_content = FileUtils.read_file(self.args.file)
        md5 = hashlib.md5(file_content.encode("utf-8")).hexdigest()
        file_name = os.path.basename(self.args.file)

        codes = self.get_edits(content)
        changes_to_make = []
        changes_made = False
        unmerged_blocks = []
        merged_blocks = []

        # First, check if there are any changes to be made
        file_content_mapping = {}
        for block in codes:
            file_path, head, update = block
            if not os.path.exists(file_path):
                changes_to_make.append((file_path, None, update))
                file_content_mapping[file_path] = update
                merged_blocks.append((file_path, "", update, 1))
                changes_made = True
            else:
                if file_path not in file_content_mapping:
                    file_content_mapping[file_path] = FileUtils.read_file(file_path)
                existing_content = file_content_mapping[file_path]
                new_content = (
                    existing_content.replace(head, update, 1)
                    if head
                    else existing_content + "\n" + update
                )
                if new_content != existing_content:
                    changes_to_make.append(
                        (file_path, existing_content, new_content))
                    file_content_mapping[file_path] = new_content
                    merged_blocks.append((file_path, head, update, 1))
                    changes_made = True
                else:
                    # If the SEARCH BLOCK is not found exactly, then try to use
                    # the similarity ratio to find the best matching block
                    similarity, best_window = TextSimilarity(
                        head, existing_content
                    ).get_best_matching_window()
                    if similarity > self.args.editblock_similarity:
                        new_content = existing_content.replace(
                            best_window, update, 1)
                        if new_content != existing_content:
                            changes_to_make.append(
                                (file_path, existing_content, new_content)
                            )
                            file_content_mapping[file_path] = new_content
                            merged_blocks.append(
                                (file_path, head, update, similarity))
                            changes_made = True
                    else:
                        unmerged_blocks.append(
                            (file_path, head, update, similarity))

        if unmerged_blocks:                        
            self.printer.print_in_terminal("unmerged_blocks_warning", num_blocks=len(unmerged_blocks))
            self._print_unmerged_blocks(unmerged_blocks)
            return
        
        # 收集新创建的文件
        new_files = []
        for file_path in file_content_mapping.keys():
            if not os.path.exists(file_path):
                new_files.append(file_path)
        
        # 在 commit 之前，如果有变更且有新文件，添加到 memory_manager
        if changes_made and new_files:
            memory_manager = get_memory_manager(self.args.source_dir)
            memory_manager.add_files(new_files)
        
        if changes_made and not force_skip_git and not self.args.skip_commit:
            try:
                git_utils.commit_changes(
                    self.args.source_dir, f"auto_coder_pre_{file_name}_{md5}"
                )
            except Exception as e:
                self.printer.print_str_in_terminal(
                    self.git_require_msg(source_dir=self.args.source_dir, error=str(e)),
                    style="red"
                )
                return
            
        # Now, apply the changes
        for file_path, new_content in file_content_mapping.items():
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            with open(file_path, "w") as f:
                f.write(new_content)

        if changes_made:
            if not force_skip_git and not self.args.skip_commit:
                try:
                    commit_result = git_utils.commit_changes(
                        self.args.source_dir,
                        f"{self.args.query}\nauto_coder_{file_name}",
                    )
                    
                    action_yml_file_manager = ActionYmlFileManager(self.args.source_dir)
                    action_file_name = os.path.basename(self.args.file)
                    add_updated_urls = []
                    commit_result.changed_files
                    for file in commit_result.changed_files:
                        add_updated_urls.append(os.path.join(self.args.source_dir, file))

                    self.args.add_updated_urls = add_updated_urls
                    update_yaml_success = action_yml_file_manager.update_yaml_field(action_file_name, "add_updated_urls", add_updated_urls)
                    if not update_yaml_success:                        
                        self.printer.print_in_terminal("yaml_save_error", style="red", yaml_file=action_file_name)  

                    if self.args.enable_active_context:
                        active_context_manager = ActiveContextManager(self.llm, self.args.source_dir)
                        task_id = active_context_manager.process_changes(self.args)
                        self.printer.print_in_terminal("active_context_background_task", 
                                                     style="blue",
                                                     task_id=task_id)
                    git_utils.print_commit_info(commit_result=commit_result)
                except Exception as e:
                    self.printer.print_str_in_terminal(
                        self.git_require_msg(source_dir=self.args.source_dir, error=str(e)),
                        style="red"
                    )
            else:
                self.print_merged_blocks(merged_blocks)
            
            self.printer.print_in_terminal("merge_success", 
                                         num_files=len(file_content_mapping.keys()),
                                         num_changes=len(changes_to_make),
                                         total_blocks=len(codes))
            
        else:
            self.printer.print_in_terminal("no_changes_made")

    def _print_unmerged_blocks(self, unmerged_blocks: List[tuple]):
        self.printer.print_in_terminal("unmerged_blocks_title", style="bold red")
        for file_path, head, update, similarity in unmerged_blocks:
            self.printer.print_str_in_terminal(
                f"\n{self.printer.get_message_from_key_with_format('unmerged_file_path',file_path=file_path)}",
                style="bold blue"
            )
            self.printer.print_str_in_terminal(
                f"\n{self.printer.get_message_from_key_with_format('unmerged_search_block',similarity=similarity)}",
                style="bold green"
            )
            syntax = Syntax(head, "python", theme="monokai", line_numbers=True)
            self.printer.console.print(Panel(syntax, expand=False))
            self.printer.print_in_terminal("unmerged_replace_block", style="bold yellow")
            syntax = Syntax(update, "python", theme="monokai", line_numbers=True)
            self.printer.console.print(Panel(syntax, expand=False))
        self.printer.print_in_terminal("unmerged_blocks_total", num_blocks=len(unmerged_blocks), style="bold red")


    def print_merged_blocks(self, merged_blocks: List[tuple]):
        """Print search/replace blocks for user review using rich library"""
        from rich.syntax import Syntax
        from rich.panel import Panel

        # Group blocks by file path
        file_blocks = {}
        for file_path, head, update, similarity in merged_blocks:
            if file_path not in file_blocks:
                file_blocks[file_path] = []
            file_blocks[file_path].append((head, update, similarity))

        # Generate formatted text for each file
        formatted_text = ""
        for file_path, blocks in file_blocks.items():
            formatted_text += f"##File: {file_path}\n"
            for head, update, similarity in blocks:
                formatted_text += "<<<<<<< SEARCH\n"
                formatted_text += head + "\n"
                formatted_text += "=======\n"
                formatted_text += update + "\n"
                formatted_text += ">>>>>>> REPLACE\n"
            formatted_text += "\n"

        # Print with rich panel
        self.printer.print_in_terminal("merged_blocks_title", style="bold green")
        self.printer.console.print(
            Panel(
                Syntax(formatted_text, "diff", theme="monokai"),
                title="Merged Changes",
                border_style="green",
                expand=False
            )
        )



"""
Test suite for LLM Friendly Package Manager

This module provides comprehensive tests for all package manager functionality
including library management, documentation handling, repository operations,
and integration with core_config.
"""

import os
import tempfile
import shutil
import pytest
from unittest.mock import Mock, patch, MagicMock
from pathlib import Path

from autocoder.common.llm_friendly_package import (
    LLMFriendlyPackageManager,
    get_package_manager,
    LibraryInfo,
    RepositoryInfo,
    PackageDoc
)


class TestLLMFriendlyPackageManager:
    """Test suite for LLMFriendlyPackageManager"""
    
    @pytest.fixture
    def temp_project_root(self):
        """Create a temporary project root directory"""
        temp_dir = tempfile.mkdtemp()
        yield temp_dir
        shutil.rmtree(temp_dir, ignore_errors=True)
    
    @pytest.fixture
    def mock_memory_manager(self):
        """Mock memory manager for testing"""
        mock_manager = Mock()
        mock_manager.get_libs.return_value = {}
        mock_manager.get_config.return_value = "https://github.com/allwefantasy/llm_friendly_packages"
        mock_manager.has_lib.return_value = False
        mock_manager.add_lib.return_value = None
        mock_manager.remove_lib.return_value = True
        mock_manager.set_config.return_value = None
        return mock_manager
    
    @pytest.fixture
    def manager(self, temp_project_root, mock_memory_manager):
        """Create a package manager instance for testing"""
        with patch('autocoder.common.llm_friendly_package.main_manager.get_memory_manager') as mock_get_memory:
            mock_get_memory.return_value = mock_memory_manager
            return LLMFriendlyPackageManager(temp_project_root)
    
    @pytest.fixture
    def mock_repo_structure(self, manager):
        """Create a mock repository structure for testing"""
        repo_dir = manager.llm_friendly_packages_dir
        os.makedirs(repo_dir, exist_ok=True)
        
        # Create test library structure
        test_lib_path = os.path.join(repo_dir, "github.com", "testuser", "testlib")
        os.makedirs(test_lib_path, exist_ok=True)
        
        # Create test markdown file
        readme_path = os.path.join(test_lib_path, "README.md")
        with open(readme_path, "w") as f:
            f.write("# Test Library\n\nThis is a test library for LLM.")
        
        # Create another test library
        test_lib2_path = os.path.join(repo_dir, "github.com", "testuser", "testlib2")
        os.makedirs(test_lib2_path, exist_ok=True)
        
        docs_path = os.path.join(test_lib2_path, "docs.md")
        with open(docs_path, "w") as f:
            f.write("# Test Library 2 Documentation\n\nDetailed docs here.")
        
        return repo_dir
    
    def test_manager_initialization(self, temp_project_root, mock_memory_manager):
        """Test manager initialization"""
        with patch('autocoder.common.llm_friendly_package.main_manager.get_memory_manager') as mock_get_memory:
            mock_get_memory.return_value = mock_memory_manager
            
            manager = LLMFriendlyPackageManager(temp_project_root)
            
            assert manager.project_root == temp_project_root
            assert manager.memory_manager == mock_memory_manager
            assert manager.lib_dir == os.path.join(temp_project_root, ".auto-coder", "libs")
            assert os.path.exists(manager.lib_dir)
    
    def test_get_package_manager_function(self, temp_project_root, mock_memory_manager):
        """Test the convenience function"""
        with patch('autocoder.common.llm_friendly_package.main_manager.get_memory_manager') as mock_get_memory:
            mock_get_memory.return_value = mock_memory_manager
            
            manager = get_package_manager(temp_project_root)
            assert isinstance(manager, LLMFriendlyPackageManager)
            assert manager.project_root == temp_project_root
    
    def test_default_proxy_url(self, manager):
        """Test default proxy URL"""
        assert manager.default_proxy_url == "https://github.com/allwefantasy/llm_friendly_packages"
    
    def test_list_added_libraries_empty(self, manager):
        """Test listing libraries when none are added"""
        libs = manager.list_added_libraries()
        assert libs == []
    
    def test_list_added_libraries_with_data(self, manager, mock_memory_manager):
        """Test listing libraries with data"""
        mock_memory_manager.get_libs.return_value = {"numpy": {}, "pandas": {}}
        
        libs = manager.list_added_libraries()
        assert set(libs) == {"numpy", "pandas"}
    
    def test_add_library_success(self, manager, mock_memory_manager):
        """Test successful library addition"""
        mock_memory_manager.has_lib.return_value = False
        
        with patch.object(manager, '_clone_repository', return_value=True):
            result = manager.add_library("test-lib")
            
            assert result is True
            mock_memory_manager.add_lib.assert_called_once_with("test-lib", {})
    
    def test_add_library_already_exists(self, manager, mock_memory_manager):
        """Test adding library that already exists"""
        mock_memory_manager.has_lib.return_value = True
        
        with patch.object(manager, '_clone_repository', return_value=True):
            result = manager.add_library("existing-lib")
            
            assert result is False
            mock_memory_manager.add_lib.assert_not_called()
    
    def test_add_library_clone_fails(self, manager, mock_memory_manager):
        """Test library addition when repository clone fails"""
        mock_memory_manager.has_lib.return_value = False
        
        with patch.object(manager, '_clone_repository', return_value=False):
            result = manager.add_library("test-lib")
            
            assert result is False
            mock_memory_manager.add_lib.assert_not_called()
    
    def test_remove_library_success(self, manager, mock_memory_manager):
        """Test successful library removal"""
        mock_memory_manager.has_lib.return_value = True
        
        result = manager.remove_library("test-lib")
        
        assert result is True
        mock_memory_manager.remove_lib.assert_called_once_with("test-lib")
    
    def test_remove_library_not_found(self, manager, mock_memory_manager):
        """Test removing library that doesn't exist"""
        mock_memory_manager.has_lib.return_value = False
        
        result = manager.remove_library("nonexistent-lib")
        
        assert result is False
        mock_memory_manager.remove_lib.assert_not_called()
    
    def test_has_library(self, manager, mock_memory_manager):
        """Test checking if library exists"""
        mock_memory_manager.has_lib.side_effect = lambda name: name == "existing-lib"
        
        assert manager.has_library("existing-lib") is True
        assert manager.has_library("nonexistent-lib") is False
    
    def test_list_all_available_libraries_no_repo(self, manager):
        """Test listing available libraries when repository doesn't exist"""
        libs = manager.list_all_available_libraries()
        assert libs == []
    
    def test_list_all_available_libraries_with_repo(self, manager, mock_repo_structure, mock_memory_manager):
        """Test listing available libraries with repository"""
        mock_memory_manager.get_libs.return_value = {"testlib": {}}
        
        libs = manager.list_all_available_libraries()
        
        assert len(libs) == 2
        
        # Check first library
        lib1 = next(lib for lib in libs if lib.lib_name == "testlib")
        assert lib1.domain == "github.com"
        assert lib1.username == "testuser"
        assert lib1.lib_name == "testlib"
        assert lib1.full_path == "testuser/testlib"
        assert lib1.is_added is True
        assert lib1.has_md_files is True
        
        # Check second library
        lib2 = next(lib for lib in libs if lib.lib_name == "testlib2")
        assert lib2.is_added is False
    
    def test_get_package_path_found(self, manager, mock_repo_structure):
        """Test getting package path when package exists"""
        path = manager.get_package_path("testlib")
        expected_path = os.path.join(mock_repo_structure, "github.com", "testuser", "testlib")
        assert path == expected_path
        
        # Test with full path format
        path2 = manager.get_package_path("testuser/testlib")
        assert path2 == expected_path
    
    def test_get_package_path_not_found(self, manager, mock_repo_structure):
        """Test getting package path when package doesn't exist"""
        path = manager.get_package_path("nonexistent")
        assert path is None
    
    def test_get_package_path_no_repo(self, manager):
        """Test getting package path when repository doesn't exist"""
        path = manager.get_package_path("testlib")
        assert path is None
    
    def test_get_docs_no_repo(self, manager):
        """Test getting docs when repository doesn't exist"""
        docs = manager.get_docs()
        assert docs == []
    
    def test_get_docs_content(self, manager, mock_repo_structure, mock_memory_manager):
        """Test getting documentation content"""
        mock_memory_manager.get_libs.return_value = {"testlib": {}, "testlib2": {}}
        
        # Get all docs content
        docs = manager.get_docs(return_paths=False)
        assert len(docs) == 2
        assert "# Test Library" in docs[0]
        assert "# Test Library 2 Documentation" in docs[1]
        
        # Get specific package docs
        specific_docs = manager.get_docs("testlib", return_paths=False)
        assert len(specific_docs) == 1
        assert "# Test Library" in specific_docs[0]
    
    def test_get_docs_paths(self, manager, mock_repo_structure, mock_memory_manager):
        """Test getting documentation paths"""
        mock_memory_manager.get_libs.return_value = {"testlib": {}, "testlib2": {}}
        
        # Get all docs paths
        paths = manager.get_docs(return_paths=True)
        assert len(paths) == 2
        assert all(path.endswith(".md") for path in paths)
        
        # Get specific package paths
        specific_paths = manager.get_docs("testlib", return_paths=True)
        assert len(specific_paths) == 1
        assert specific_paths[0].endswith("README.md")
    
    def test_get_library_docs_paths(self, manager, mock_repo_structure, mock_memory_manager):
        """Test getting library documentation paths"""
        mock_memory_manager.get_libs.return_value = {"testlib": {}}
        
        paths = manager.get_library_docs_paths("testlib")
        assert len(paths) == 1
        assert paths[0].endswith("README.md")
    
    def test_get_library_docs_content(self, manager, mock_repo_structure, mock_memory_manager):
        """Test getting library documentation content"""
        mock_memory_manager.get_libs.return_value = {"testlib": {}}
        
        content = manager.get_library_docs_content("testlib")
        assert len(content) == 1
        assert "# Test Library" in content[0]
    
    @patch('git.Repo')
    def test_clone_repository_success(self, mock_git_repo, manager):
        """Test successful repository cloning"""
        # Repository doesn't exist
        assert not os.path.exists(manager.llm_friendly_packages_dir)
        
        result = manager._clone_repository()
        
        assert result is True
        mock_git_repo.clone_from.assert_called_once()
    
    @patch('git.Repo')
    def test_clone_repository_already_exists(self, mock_git_repo, manager):
        """Test cloning when repository already exists"""
        # Create the repository directory
        os.makedirs(manager.llm_friendly_packages_dir)
        
        result = manager._clone_repository()
        
        assert result is True
        mock_git_repo.clone_from.assert_not_called()
    
    @patch('git.Repo')
    def test_clone_repository_failure(self, mock_git_repo, manager):
        """Test repository cloning failure"""
        from git import GitCommandError
        mock_git_repo.clone_from.side_effect = GitCommandError("clone", "Error")
        
        result = manager._clone_repository()
        
        assert result is False
    
    @patch('git.Repo')
    def test_refresh_repository_success(self, mock_git_repo, manager):
        """Test successful repository refresh"""
        # Create repository directory
        os.makedirs(manager.llm_friendly_packages_dir)
        
        # Mock repository
        mock_repo = Mock()
        mock_origin = Mock()
        mock_origin.url = "https://github.com/allwefantasy/llm_friendly_packages"
        mock_repo.remotes.origin = mock_origin
        mock_git_repo.return_value = mock_repo
        
        result = manager.refresh_repository()
        
        assert result is True
        mock_origin.pull.assert_called_once()
    
    def test_refresh_repository_no_repo(self, manager):
        """Test refreshing when repository doesn't exist"""
        result = manager.refresh_repository()
        assert result is False
    
    @patch('git.Repo')
    def test_refresh_repository_failure(self, mock_git_repo, manager):
        """Test repository refresh failure"""
        from git import GitCommandError
        
        # Create repository directory
        os.makedirs(manager.llm_friendly_packages_dir)
        
        mock_repo = Mock()
        mock_origin = Mock()
        mock_origin.pull.side_effect = GitCommandError("pull", "Error")
        mock_repo.remotes.origin = mock_origin
        mock_git_repo.return_value = mock_repo
        
        result = manager.refresh_repository()
        assert result is False
    
    def test_set_proxy_get_current(self, manager, mock_memory_manager):
        """Test getting current proxy URL"""
        current_proxy = manager.set_proxy()
        expected_url = "https://github.com/allwefantasy/llm_friendly_packages"
        assert current_proxy == expected_url
        mock_memory_manager.get_config.assert_called_with("lib-proxy", expected_url)
    
    def test_set_proxy_new_url(self, manager, mock_memory_manager):
        """Test setting new proxy URL"""
        new_url = "https://custom-proxy.com/repo"
        result = manager.set_proxy(new_url)
        
        assert result == new_url
        mock_memory_manager.set_config.assert_called_once_with("lib-proxy", new_url)
    
    def test_get_repository_info_exists(self, manager, mock_repo_structure):
        """Test getting repository info when it exists"""
        with patch('git.Repo') as mock_git_repo:
            # Mock commit
            mock_commit = Mock()
            mock_commit.committed_datetime.isoformat.return_value = "2023-01-01T12:00:00"
            
            mock_repo = Mock()
            mock_repo.head.commit = mock_commit
            mock_git_repo.return_value = mock_repo
            
            repo_info = manager.get_repository_info()
            
            assert repo_info.exists is True
            assert repo_info.url == "https://github.com/allwefantasy/llm_friendly_packages"
            assert repo_info.last_updated == "2023-01-01T12:00:00"
    
    def test_get_repository_info_not_exists(self, manager):
        """Test getting repository info when it doesn't exist"""
        repo_info = manager.get_repository_info()
        
        assert repo_info.exists is False
        assert repo_info.url == "https://github.com/allwefantasy/llm_friendly_packages"
        assert repo_info.last_updated is None
    
    def test_display_added_libraries_empty(self, manager, capsys):
        """Test displaying empty library list"""
        manager.display_added_libraries()
        captured = capsys.readouterr()
        assert "No libraries added yet" in captured.out
    
    def test_display_added_libraries_with_data(self, manager, mock_memory_manager, capsys):
        """Test displaying library list with data"""
        mock_memory_manager.get_libs.return_value = {"numpy": {}, "pandas": {}}
        
        manager.display_added_libraries()
        captured = capsys.readouterr()
        # Console output includes library names in the table
        assert "numpy" in captured.out or "pandas" in captured.out
    
    def test_display_all_libraries_no_repo(self, manager, capsys):
        """Test displaying all libraries when repository doesn't exist"""
        manager.display_all_libraries()
        captured = capsys.readouterr()
        assert "repository does not exist" in captured.out
    
    def test_display_library_docs_no_docs(self, manager, capsys):
        """Test displaying library docs when none exist"""
        manager.display_library_docs("nonexistent")
        captured = capsys.readouterr()
        assert "No markdown files found" in captured.out


class TestDataModels:
    """Test suite for data models"""
    
    def test_library_info_creation(self):
        """Test LibraryInfo creation"""
        lib_info = LibraryInfo(
            domain="github.com",
            username="testuser",
            lib_name="testlib",
            full_path="testuser/testlib",
            is_added=True,
            has_md_files=True
        )
        
        assert lib_info.domain == "github.com"
        assert lib_info.username == "testuser"
        assert lib_info.lib_name == "testlib"
        assert lib_info.full_path == "testuser/testlib"
        assert lib_info.is_added is True
        assert lib_info.has_md_files is True
    
    def test_library_info_defaults(self):
        """Test LibraryInfo default values"""
        lib_info = LibraryInfo(
            domain="github.com",
            username="testuser",
            lib_name="testlib",
            full_path="testuser/testlib",
            is_added=False
        )
        
        assert lib_info.has_md_files is True  # Default value
    
    def test_package_doc_creation(self):
        """Test PackageDoc creation"""
        doc = PackageDoc(
            file_path="/path/to/doc.md",
            content="# Documentation"
        )
        
        assert doc.file_path == "/path/to/doc.md"
        assert doc.content == "# Documentation"
    
    def test_package_doc_no_content(self):
        """Test PackageDoc without content"""
        doc = PackageDoc(file_path="/path/to/doc.md")
        
        assert doc.file_path == "/path/to/doc.md"
        assert doc.content is None
    
    def test_repository_info_creation(self):
        """Test RepositoryInfo creation"""
        repo_info = RepositoryInfo(
            exists=True,
            url="https://github.com/test/repo",
            last_updated="2023-01-01T12:00:00"
        )
        
        assert repo_info.exists is True
        assert repo_info.url == "https://github.com/test/repo"
        assert repo_info.last_updated == "2023-01-01T12:00:00"
    
    def test_repository_info_defaults(self):
        """Test RepositoryInfo default values"""
        repo_info = RepositoryInfo(
            exists=False,
            url="https://github.com/test/repo"
        )
        
        assert repo_info.exists is False
        assert repo_info.url == "https://github.com/test/repo"
        assert repo_info.last_updated is None


class TestIntegration:
    """Integration tests"""
    
    @pytest.fixture
    def real_temp_project(self):
        """Create a real temporary project for integration testing"""
        temp_dir = tempfile.mkdtemp()
        yield temp_dir
        shutil.rmtree(temp_dir, ignore_errors=True)
    
    def test_full_workflow_with_mocked_git(self, real_temp_project):
        """Test a complete workflow with mocked git operations"""
        with patch('git.Repo') as mock_git_repo:
            # Create manager
            manager = LLMFriendlyPackageManager(real_temp_project)
            
            # Add a library (this will trigger repository cloning)
            result = manager.add_library("test-library")
            assert result is True
            
            # Check that library is added
            assert manager.has_library("test-library")
            libs = manager.list_added_libraries()
            assert "test-library" in libs
            
            # Remove the library
            result = manager.remove_library("test-library")
            assert result is True
            assert not manager.has_library("test-library")
    
    def test_error_handling_in_docs_reading(self, real_temp_project):
        """Test error handling when reading documentation files"""
        manager = LLMFriendlyPackageManager(real_temp_project)
        
        # Create a corrupted file structure
        repo_dir = manager.llm_friendly_packages_dir
        os.makedirs(repo_dir, exist_ok=True)
        
        test_lib_path = os.path.join(repo_dir, "github.com", "testuser", "testlib")
        os.makedirs(test_lib_path, exist_ok=True)
        
        # Create a file with permission issues (simulate by creating a directory with same name)
        bad_file_path = os.path.join(test_lib_path, "README.md")
        os.makedirs(bad_file_path, exist_ok=True)  # This will cause read error
        
        # Add the library
        manager.add_library("testlib")
        
        # Try to read docs - should handle the error gracefully
        docs = manager.get_library_docs_content("testlib")
        assert docs == []  # Should return empty list on error


if __name__ == "__main__":
    # Run tests when script is executed directly
    pytest.main([__file__, "-v"]) 


import os
from typing import List, Callable, Optional,Tuple
import pdf2image
from PIL import Image
import byzerllm
from autocoder.common import AutoCoderArgs
from loguru import logger
import platform

from docx import Document
from reportlab.lib.pagesizes import letter
from reportlab.platypus import SimpleDocTemplate, Paragraph, Table, TableStyle
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib import colors

from PIL import Image, ImageDraw, ImageFont
import os
import base64
import json
from byzerllm.utils.client import code_utils
from byzerllm.apps.utils import TagExtractor

# Importing required libraries for docx conversion
try:
    import docx2pdf
except ImportError:
    docx2pdf = None

# Importing required libraries for Linux conversion
try:
    import uno
    from com.sun.star.beans import PropertyValue
    from com.sun.star.connection import NoConnectException
    from com.sun.star.task import ErrorCodeIOException
    from com.sun.star.uno import Exception as UnoException
    from com.sun.star.uno import RuntimeException
except ImportError:
    uno = None

# Importing required library for docx to pdf conversion using pypandoc
try:
    import pypandoc
except ImportError:
    pypandoc = None


class Anything2Images:
    def __init__(
        self,
        llm: byzerllm.ByzerLLM,
        args: AutoCoderArgs,
        keep_conversion: bool = False,
        continue_prompt: str = "接着前面的内容继续",
        max_steps: int = 20,
    ):
        self.llm = llm
        self.vl_model = llm.get_sub_client("vl_model")
        self.args = args
        self.output_dir = args.output
        os.makedirs(self.output_dir, exist_ok=True)
        self.keep_conversion = keep_conversion
        self.continue_prompt = continue_prompt
        self.max_steps = max_steps

    def convert(self, file_path: str) -> List[str]:
        file_path = os.path.abspath(file_path)
        if file_path.lower().endswith(".pdf"):
            return self.convert_pdf(file_path)
        elif file_path.lower().endswith(".docx"):
            return self.convert_docx(file_path)
        else:
            raise ValueError(f"Unsupported file format: {file_path}")

    @byzerllm.prompt()
    def single_file_html_prompt(self) -> str:
        """
        将图片里的内容以 HTML 格式进行输出。确保信息完整，并且请只返回以<html></html>为标签的内容，
        不要在开头或结尾包含markdown "```" 或 "```html"。
        """

    @byzerllm.prompt()
    def html_prompt(self) -> str:
        """
        回顾前面所有图片，将图片里的内容以 HTML 格式进行输出。确保信息完整，并且请只返回以<html></html>为标签的内容，
        不要在开头或结尾包含markdown "```" 或 "```html"。
        """

    def merge_table_html_prompt(self, html: str) -> str:
        """
        下面是一个 HTML 内容:

        {{ html }}


        里面有一些信息诸如表格等因为分页等原因被分割开了，请将被分割的信息做适当的合并，重新
        生成一份完整的的HTML内容。请只返回以<html></html>为标签的内容，不要在开头或结尾包含markdown "```" 或 "```html"。
        请确保HTML的完整性，而不要只生成修改部分。
        """

    def _save_conversation(self, conversations):
        if self.keep_conversion:
            with open(
                os.path.join(self.output_dir, "conversations.json"),
                "w",
                encoding="utf-8",
            ) as f:
                f.write((json.dumps(conversations, ensure_ascii=False, indent=4)))

    def to_html_from_images(self, images: List[str]) -> str:
        conversations = []
        if not self.args.single_file:
            for i, image in enumerate(images):
                img_path = image
                image_path_ext = os.path.splitext(img_path)[1][1:]
                with open(img_path, "rb") as image_file:
                    image = base64.b64encode(image_file.read()).decode("utf-8")
                    image = f"data:image/{image_path_ext};base64,{image}"

                conversations.append(
                    {
                        "role": "user",
                        "content": json.dumps(
                            [
                                {
                                    "image": image,
                                    "detail": "high",
                                    "text": f"当你看到这张图片的时候，请回复'收到'",
                                }
                            ],
                            ensure_ascii=False,
                        ),
                    }
                )
                # t = self.vl_model.chat_oai(conversations=conversations)
                conversations.append(
                    {
                        "role": "assistant",
                        "content": json.dumps([{"text": "收到"}], ensure_ascii=False),
                    }
                )
                logger.info(f"Collected {i}:{img_path}")

            logger.info("All images are collected. Now start to generate html.")
            conversations.append(
                {
                    "role": "user",
                    "content": json.dumps(
                        [{"text": self.html_prompt.prompt()}], ensure_ascii=False
                    ),
                }
            )
            t = self.vl_model.chat_oai(conversations=conversations)
            conversations.append(
                {
                    "role": "assistant",
                    "content": json.dumps([{"text": t[0].output}], ensure_ascii=False),
                }
            )
            html = t[0].output
        else:
            img_path = images[0]
            image_path_ext = os.path.splitext(img_path)[1][1:]
            with open(img_path, "rb") as image_file:
                image = base64.b64encode(image_file.read()).decode("utf-8")
                image = f"data:image/{image_path_ext};base64,{image}"
            conversations.append(
                {
                    "role": "user",
                    "content": json.dumps(
                        [
                            {
                                "image": image,
                                "detail": "high",
                                "text": self.single_file_html_prompt.prompt(),
                            }
                        ],
                        ensure_ascii=False,
                    ),
                }
            )
            t = self.vl_model.chat_oai(conversations=conversations)
            html = t[0].output

        counter = self.max_steps

        def not_end(_html):
            _not_end = False
            if "```html" in _html:
                _not_end = not "</html>" in _html
            else:
                _not_end = not _html.strip().endswith("</html>")
            return _not_end

        while not_end(html) and counter > 0:
            counter -= 1
            conversations.append(
                {
                    "role": "user",
                    "content": json.dumps(
                        [{"text": self.continue_prompt}], ensure_ascii=False
                    ),
                }
            )
            t = self.vl_model.chat_oai(conversations=conversations)
            logger.info(
                f"The output is not finished yet. Continue to get more output. {counter}th time."
            )
            conversations.append(
                {
                    "role": "assistant",
                    "content": json.dumps([{"text": t[0].output}], ensure_ascii=False),
                }
            )
            self._save_conversation(conversations)
            html += t[0].output

        if "```html" in html:
            for lang, code in code_utils.extract_code(html):
                if lang == "html":
                    html = code
                    break
        logger.info(f"Get the final output: {html[:100]}...{html[-100:]}")
        self._save_conversation(conversations)
        return html

    @byzerllm.prompt()
    def pdf_generation_prompt(
        self, html: str, target_file_path: str, code_prefix: str
    ) -> str:
        """
        根据提供的图片和HTML内容，生成使用reportlab创建PDF的Python代码。

        HTML内容：
        {{ html }}

        你生成的PDF路径需要为： {{ target_file_path }}

        请分析图片布局和HTML内容，然后生成使用reportlab创建PDF的Python代码：

        1. 代码应该包含必要的导入语句，并使用reportlab的组件（如Canvas, Paragraph, Table等）来创建与原始文档布局相似的PDF。
        2. 请确保代码是完整的、可执行的，并包含适当的注释。
        3. 使用最新 reportlab 版本的 API
        4. 生成的代码使用 ```python ``` 包裹
        5. 生成的代码里内容确保不要遗漏任何 HTML 里的文字信息。

        reportlab 使用时需要注意：
        1. 不要使用 Link ,可以直接使用Paragraph,比如 link_text = Paragraph('<a href="http://www.example.com">Click here</a>', style)
        2. 如果使用 styles = getSampleStyleSheet()，那么不要再使用styles.add(ParagraphStyle(name='Heading1', fontSize=18, leading=22, spaceAfter=12)) 之类的，而是修改styles，比如styles['Heading1'].fontSize = 18

        下面是我们给你提供的字体，我们会将这段代码放到你生成的代码的最前面，你可以直接使用这些字体：

        ```python
        {{ code_prefix }}
        ```
        """

    @byzerllm.prompt()
    def error_retry_prompt(self, code: str, error: str, context: str) -> str:
        """

        当执行下列代码时：

        ```python
        {{ code }}
        ```

        代码执行失败，错误如下：

        ```text
        {{ error }}
        ```

        下面是原始代码原来的的需求：

        ```text
        {{ context }}
        ```

        现在根据错误修改代码，然后输出完整的修改后代码,注意，新输出的代码是需要可以直接执行而无需再次修改的。
        输出的代码使用 ```python ``` 包裹。
        """

    def run_python_code(self, code: str, context: str, retries: int = 3):
        from autocoder.common.JupyterClient import JupyterNotebook

        def run_code(code: str) -> str:
            jupyter_client = JupyterNotebook()
            try:
                return jupyter_client.add_and_run(code)
            finally:
                jupyter_client.close()

        result, error = run_code(code)
        print(result)
        print(error)
        while error and retries > 0:
            retries -= 1
            prompt = self.error_retry_prompt.prompt(
                code=code, error=error, context=context
            )
            print(prompt)
            t = self.vl_model.chat_oai(
                conversations=[{"role": "user", "content": prompt}]
            )
            code = code_utils.extract_code(t[0].output)[0][1]
            result, error = run_code(code)
            print(result)
            print(error)

    def anything_to_pdf(
        self,
        origin_file_path: str,
        target_file_path: str,
        prompt_function: Optional[Callable[[str], str]] = None,
        extractor: Optional[Callable[[str], str]] = None,
    ) -> str:
        from rich.console import Console
        from rich.panel import Panel
        from rich.syntax import Syntax

        console = Console()

        images = self.convert(origin_file_path)

        html = self.to_html_from_images(images)
        with open(
            os.path.join(self.output_dir, "output.html"), "w", encoding="utf-8"
        ) as f:
            f.write(html)

        if prompt_function:
            logger.info("processing html with prompt function")
            html = (
                prompt_function.with_llm(self.vl_model)
                .with_extractor(extractor)
                .run(html)
            )
            with open(
                os.path.join(self.output_dir, "output_prompted.html"),
                "w",
                encoding="utf-8",
            ) as f:
                f.write(html)

        @byzerllm.prompt()
        def generate_prefix_code(fonts: List[Tuple[str, str]]) -> str:
            """
            from reportlab.pdfbase import pdfmetrics
            from reportlab.pdfbase.ttfonts import TTFont
            try:
            {{ fonts}}
            except Exception as e:
                print(f"fail to load font")
            """
            fonts_str = (
                "\n".join(
                    [
                        f"    pdfmetrics.registerFont(TTFont('{font[0]}', '{font[1]}'))"
                        for font in fonts
                    ]
                )
                if fonts
                else "    pass"
            )
            return {"fonts": fonts_str}

        ttf_files = [f for f in os.listdir(self.output_dir) if f.endswith(".ttf")]
        fonts = [
            (os.path.splitext(f)[0], os.path.join(self.output_dir, f))
            for f in ttf_files
        ]
        prefix = generate_prefix_code.prompt(fonts=fonts)

        # 使用大模型生成PDF生成代码
        generate_code_prompt = self.pdf_generation_prompt.prompt(
            html=html, target_file_path=target_file_path, code_prefix=prefix
        )

        conversations = []
        if not self.args.single_file:
            for i, image in enumerate(images):
                img_path = image
                image_path_ext = os.path.splitext(img_path)[1][1:]
                with open(img_path, "rb") as image_file:
                    image = base64.b64encode(image_file.read()).decode("utf-8")
                    image = f"data:image/{image_path_ext};base64,{image}"

                conversations.append(
                    {
                        "role": "user",
                        "content": json.dumps(
                            [
                                {
                                    "image": image,
                                    "detail": "high",
                                    "text": f"当你看到这张图片的时候，请回复'收到'",
                                }
                            ],
                            ensure_ascii=False,
                        ),
                    }
                )
                conversations.append(
                    {
                        "role": "assistant",
                        "content": json.dumps([{"text": "收到"}], ensure_ascii=False),
                    }
                )
                logger.info(f"Collected {i}:{img_path}")

            logger.info("All images are collected. Now start to generate code.")
            conversations.append(
                {
                    "role": "user",
                    "content": json.dumps(
                        [{"text": generate_code_prompt}], ensure_ascii=False
                    ),
                }
            )
            t = self.vl_model.chat_oai(conversations=conversations)
            conversations.append(
                {
                    "role": "assistant",
                    "content": json.dumps([{"text": t[0].output}], ensure_ascii=False),
                }
            )
            code = t[0].output
        else:
            img_path = images[0]
            image_path_ext = os.path.splitext(img_path)[1][1:]
            with open(img_path, "rb") as image_file:
                image = base64.b64encode(image_file.read()).decode("utf-8")
                image = f"data:image/{image_path_ext};base64,{image}"
            conversations.append(
                {
                    "role": "user",
                    "content": json.dumps(
                        [
                            {
                                "image": image,
                                "detail": "high",
                                "text": generate_code_prompt,
                            }
                        ],
                        ensure_ascii=False,
                    ),
                }
            )
            t = self.vl_model.chat_oai(conversations=conversations)
            code = t[0].output

        ## 字体下载
        self.download_font(
            "https://github.com/StellarCN/scp_zh/raw/master/fonts/SimSun.ttf",
            "SimSun.ttf",
        )
        self.download_font(
            "https://github.com/hyoshiok/ttf-ipafont/raw/master/ipag.ttf", "ipag.ttf"
        )

        codes = code_utils.extract_code(code)
        pdf_generation_code = codes[0][1]
        pdf_generation_code = prefix + "\n\n" + pdf_generation_code
        # root_tag = TagExtractor(code).extract()
        # pdf_generation_code = root_tag.content[0].content

        # if not isinstance(pdf_generation_code, str):
        #     pdf_generation_code = pdf_generation_code[0]

        # 使用Rich打印生成的代码
        syntax = Syntax(
            pdf_generation_code, "python", theme="monokai", line_numbers=True
        )
        console.print(
            Panel(syntax, title="Generated PDF Generation Code", expand=False)
        )

        self.run_python_code(pdf_generation_code, context=generate_code_prompt)
        return target_file_path

    def download_font(self, font_url: str, font_name: str):
        font_path = os.path.join(self.output_dir, font_name)
        if not os.path.exists(font_path):
            import requests

            print(f"Downloading {font_name} from {font_url}")
            response = requests.get(font_url)
            if response.status_code == 200:
                with open(font_path, "wb") as f:
                    f.write(response.content)
                print(f"SimSun.ttf downloaded successfully to {font_path}")
            else:
                print(
                    f"Failed to download SimSun.ttf. Status code: {response.status_code}"
                )

    def to_html(self, file_path: str) -> str:
        images = self.convert(file_path)
        return self.to_html_from_images(images)

    def convert_pdf(self, file_path: str) -> List[str]:
        images = pdf2image.convert_from_path(file_path)
        image_paths = []

        if self.args.single_file:
            # 合并所有图片为一张图片
            total_width = max(image.width for image in images)
            total_height = sum(image.height for image in images)
            merged_image = Image.new("RGB", (total_width, total_height))
            y_offset = 0
            for image in images:
                merged_image.paste(image, (0, y_offset))
                y_offset += image.height

            merged_image_path = os.path.join(
                self.output_dir, f"{os.path.basename(file_path)}_merged.png"
            )
            merged_image.save(merged_image_path, "PNG")
            image_paths.append(merged_image_path)
        else:
            for i, image in enumerate(images, start=1):
                image_path = os.path.join(
                    self.output_dir, f"{os.path.basename(file_path)}_page{i}.png"
                )
                image.save(image_path, "PNG")
                image_paths.append(image_path)

        return image_paths

    def get_default_chinese_font(self):

        system = platform.system()
        if system == "Windows":
            return "SimSun"  # Windows 默认中文字体
        elif system == "Darwin":  # macOS
            return "STXihei"  # macOS 默认中文字体
        elif system == "Linux":
            return "WenQuanYi Zen Hei"  # Linux 默认中文字体
        else:
            raise ValueError(f"Unsupported operating system: {system}")

    def convert_word_to_pdf_v2(self, word_file, pdf_file):
        # Load the Word document
        doc = Document(word_file)

        # Create a new PDF document
        pdf_doc = SimpleDocTemplate(pdf_file, pagesize=letter)

        # Extract the content from the Word document
        content = []
        for element in doc.body.elements:
            if isinstance(element, Document):
                # Extract paragraphs from the document
                for paragraph in element.paragraphs:
                    text = paragraph.text
                    pdf_paragraph = Paragraph(text, getSampleStyleSheet()["Normal"])
                    content.append(pdf_paragraph)
            elif isinstance(element, Document.table):
                # Extract tables from the document
                table_data = []
                for row in element.rows:
                    row_data = []
                    for cell in row.cells:
                        row_data.append(cell.text)
                    table_data.append(row_data)

                # Create a Table object with the extracted data
                pdf_table = Table(table_data)

                # Apply table styles (optional)
                table_style = TableStyle(
                    [
                        ("BACKGROUND", (0, 0), (-1, 0), colors.grey),
                        ("TEXTCOLOR", (0, 0), (-1, 0), colors.whitesmoke),
                        ("ALIGN", (0, 0), (-1, -1), "CENTER"),
                        ("FONTNAME", (0, 0), (-1, 0), "Helvetica-Bold"),
                        ("FONTSIZE", (0, 0), (-1, 0), 14),
                        ("BOTTOMPADDING", (0, 0), (-1, 0), 12),
                        ("BACKGROUND", (0, 1), (-1, -1), colors.beige),
                        ("TEXTCOLOR", (0, 1), (-1, -1), colors.black),
                        ("ALIGN", (0, 1), (-1, -1), "LEFT"),
                        ("FONTNAME", (0, 1), (-1, -1), "Helvetica"),
                        ("FONTSIZE", (0, 1), (-1, -1), 12),
                        ("TOPPADDING", (0, 1), (-1, -1), 6),
                        ("BOTTOMPADDING", (0, 1), (-1, -1), 6),
                    ]
                )
                pdf_table.setStyle(table_style)

                # Append the Table object to the content list
                content.append(pdf_table)

        # Build the PDF document
        pdf_doc.build(content)

        print("Word document converted to PDF successfully!")

    def convert_docx(self, file_path: str) -> List[str]:
        if uno:
            pdf_path = self.convert_docx_linux(file_path)
            logger(f"Converted {file_path} to {pdf_path} using LibreOffice")

        elif docx2pdf:
            pdf_path = os.path.join(
                self.output_dir, f"{os.path.basename(file_path)}.pdf"
            )
            try:
                docx2pdf.convert(file_path, pdf_path)
            except:
                logger.info("docx2pdf failed. Trying to convert using pypandoc.")

            if not os.path.exists(pdf_path):
                if pypandoc:
                    logger.info(
                        "docx2pdf failed. Trying to convert using pypandoc. Downloading pandoc..."
                    )
                    pypandoc.download_pandoc()
                    pypandoc.convert_file(
                        file_path,
                        "pdf",
                        outputfile=pdf_path,
                        extra_args=[
                            "--pdf-engine=xelatex",
                            "--variable",
                            f'mainfont="{self.get_default_chinese_font()}"',
                        ],
                    )
                    logger.info(f"Converted {file_path} to {pdf_path} using pypandoc")
                else:
                    raise ImportError(
                        "Neither docx2pdf nor pypandoc are available for DOCX conversion."
                    )

        else:
            raise ImportError(
                "Neither docx2pdf nor uno are available for DOCX conversion."
            )

        if not os.path.exists(pdf_path):
            raise RuntimeError("Failed to convert DOCX to PDF")

        image_paths = self.convert_pdf(pdf_path)
        os.remove(pdf_path)
        return image_paths

    def convert_docx_linux(self, file_path: str) -> str:
        output_pdf = os.path.join(self.output_dir, f"{os.path.basename(file_path)}.pdf")
        local_context = uno.getComponentContext()
        resolver = local_context.ServiceManager.createInstanceWithContext(
            "com.sun.star.bridge.UnoUrlResolver", local_context
        )
        try:
            ctx = resolver.resolve(
                "uno:pipe,name=officepipe;urp;StarOffice.ComponentContext"
            )
        except NoConnectException:
            raise RuntimeError(
                "LibreOffice is not running. Start it with 'libreoffice --headless --accept=\"socket,host=localhost,port=2002;urp;StarOffice.ServiceManager\"'"
            )
        desktop = ctx.ServiceManager.createInstanceWithContext(
            "com.sun.star.frame.Desktop", ctx
        )
        convert_props = (PropertyValue("FilterName", 0, "writer_pdf_Export", 0),)
        url = uno.systemPathToFileUrl(file_path)
        out_url = uno.systemPathToFileUrl(output_pdf)
        doc = desktop.loadComponentFromURL(url, "_blank", 0, ())
        doc.storeToURL(out_url, convert_props)
        doc.close(True)
        return output_pdf



"""
Diff 解析模块

提供单文件和多文件 SEARCH/REPLACE 块的纯解析功能。
所有函数均为纯函数，无 IO 副作用，便于单元测试。
"""

from dataclasses import dataclass, field
from typing import List, Tuple, Optional
from loguru import logger


# ============================================================================
# 标准标记常量
# ============================================================================

SEARCH_MARKER: str = "<<<<<<< SEARCH"
DIVIDER_MARKER: str = "======="
REPLACE_MARKER: str = ">>>>>>> REPLACE"

DEFAULT_FENCE_OPEN: str = "```"
DEFAULT_FENCE_CLOSE: str = "```"


# ============================================================================
# 数据结构
# ============================================================================


@dataclass
class SearchReplaceBlock:
    """单个 SEARCH/REPLACE 块的结构化表示。

    Attributes:
        search: 要搜索的文本（原始换行保留）
        replace: 替换后的文本（原始换行保留）
    """

    search: str
    replace: str

    def as_tuple(self) -> Tuple[str, str]:
        """兼容旧接口，返回 (search, replace) 元组。"""
        return (self.search, self.replace)


@dataclass
class MultiFileEdit:
    """多文件编辑块的结构化表示。

    Attributes:
        path: 文件路径（相对路径或绝对路径）
        blocks: 该文件的所有 SEARCH/REPLACE 块
    """

    path: str
    blocks: List[SearchReplaceBlock] = field(default_factory=list)

    def as_tuples(self) -> List[Tuple[str, str]]:
        """兼容旧接口，返回 [(search, replace), ...] 列表。"""
        return [b.as_tuple() for b in self.blocks]


# ============================================================================
# 单文件解析
# ============================================================================


def parse_search_replace_blocks(
    diff_content: str,
    search_marker: str = SEARCH_MARKER,
    divider_marker: str = DIVIDER_MARKER,
    replace_marker: str = REPLACE_MARKER,
) -> List[SearchReplaceBlock]:
    """解析单文件 diff 内容为 SEARCH/REPLACE 块列表。

    支持标准格式：
        <<<<<<< SEARCH
        [search content]
        =======
        [replace content]
        >>>>>>> REPLACE

    Args:
        diff_content: 包含 SEARCH/REPLACE 块的 diff 文本
        search_marker: SEARCH 起始标记
        divider_marker: 分隔标记
        replace_marker: REPLACE 结束标记

    Returns:
        解析出的 SearchReplaceBlock 列表（保留原始换行）
    """
    blocks: List[SearchReplaceBlock] = []
    lines = diff_content.splitlines(keepends=True)
    i = 0
    n = len(lines)

    while i < n:
        line = lines[i]
        if line.strip() == search_marker:
            i += 1
            search_lines: List[str] = []
            # 收集 SEARCH 块内容
            while i < n and lines[i].strip() != divider_marker:
                search_lines.append(lines[i])
                i += 1
            if i >= n:
                logger.warning("Unterminated SEARCH block found in diff content.")
                break
            i += 1  # 跳过分隔符

            replace_lines: List[str] = []
            # 收集 REPLACE 块内容
            while i < n and lines[i].strip() != replace_marker:
                replace_lines.append(lines[i])
                i += 1
            if i >= n:
                logger.warning("Unterminated REPLACE block found in diff content.")
                break
            i += 1  # 跳过结束标记

            blocks.append(
                SearchReplaceBlock(
                    search="".join(search_lines),
                    replace="".join(replace_lines),
                )
            )
        else:
            i += 1

    if not blocks and diff_content.strip():
        logger.warning(
            f"Could not parse any SEARCH/REPLACE blocks from diff content (length={len(diff_content)})"
        )

    return blocks


def parse_search_replace_blocks_as_tuples(
    diff_content: str,
    search_marker: str = SEARCH_MARKER,
    divider_marker: str = DIVIDER_MARKER,
    replace_marker: str = REPLACE_MARKER,
) -> List[Tuple[str, str]]:
    """解析单文件 diff 并返回元组列表（兼容旧接口）。"""
    blocks = parse_search_replace_blocks(
        diff_content, search_marker, divider_marker, replace_marker
    )
    return [b.as_tuple() for b in blocks]


# ============================================================================
# 多文件解析
# ============================================================================


@dataclass
class _RawEditBlock:
    """内部用：解析多文件格式时的原始块。"""

    path: str
    content: str  # 包含 SEARCH/REPLACE 标记的原始内容


def _parse_multi_file_raw_blocks(
    text: str,
    fence_open: str = DEFAULT_FENCE_OPEN,
    fence_close: str = DEFAULT_FENCE_CLOSE,
    search_marker: str = SEARCH_MARKER,
) -> List[_RawEditBlock]:
    """解析多文件格式的原始块（支持 two_line_mode 和 one_line_mode）。

    two_line_mode:
        ```python
        ##File: path/to/file.py
        <<<<<<< SEARCH
        ...
        ```

    one_line_mode:
        ```python:path/to/file.py
        <<<<<<< SEARCH
        ...
        ```

    Args:
        text: 包含多文件编辑块的完整文本
        fence_open: 代码块开始标记（默认 ```）
        fence_close: 代码块结束标记（默认 ```）
        search_marker: SEARCH 标记

    Returns:
        解析出的原始编辑块列表
    """
    lines = text.split("\n")
    lines_len = len(lines)
    result: List[_RawEditBlock] = []

    start_marker_count = 0
    block_lines: List[str] = []
    current_mode = "two_line_mode"
    current_path: Optional[str] = None

    def guard(index: int) -> bool:
        return index + 1 < lines_len

    def is_start_marker(line: str, index: int) -> bool:
        nonlocal current_mode, current_path

        # one_line_mode: ```lang:path/to/file
        if (
            line.startswith(fence_open)
            and guard(index)
            and ":" in line
            and lines[index + 1].startswith(search_marker)
        ):
            current_mode = "one_line_mode"
            current_path = line.split(":", 1)[1].strip()
            return True

        # two_line_mode: ``` 后跟 ##File:
        if (
            line.startswith(fence_open)
            and guard(index)
            and lines[index + 1].startswith("##File:")
        ):
            current_mode = "two_line_mode"
            current_path = None
            return True

        return False

    def is_end_marker(line: str, index: int) -> bool:
        return (
            line.startswith(fence_close)
            and index > 0
            and REPLACE_MARKER in lines[index - 1]
        )

    for index, line in enumerate(lines):
        if is_start_marker(line, index) and start_marker_count == 0:
            start_marker_count += 1
        elif is_end_marker(line, index) and start_marker_count == 1:
            start_marker_count -= 1
            if block_lines:
                if current_mode == "two_line_mode":
                    # 第一行是 ##File: path
                    path = block_lines[0].split(":", 1)[1].strip()
                    content = "\n".join(block_lines[1:])
                else:
                    path = current_path or ""
                    content = "\n".join(block_lines)
                block_lines = []
                if path:
                    result.append(_RawEditBlock(path=path, content=content))
        elif start_marker_count > 0:
            block_lines.append(line)

    return result


def _parse_block_content(
    content: str,
    search_marker: str = SEARCH_MARKER,
    divider_marker: str = DIVIDER_MARKER,
    replace_marker: str = REPLACE_MARKER,
    strip_trailing_newline: bool = True,
) -> SearchReplaceBlock:
    """解析单个编辑块的内容为 SearchReplaceBlock。

    Args:
        content: 包含 SEARCH/REPLACE 标记的块内容
        search_marker: SEARCH 标记
        divider_marker: 分隔标记
        replace_marker: REPLACE 标记
        strip_trailing_newline: 是否去除末尾换行（多文件模式默认 True）

    Returns:
        解析后的 SearchReplaceBlock
    """
    heads: List[str] = []
    updates: List[str] = []
    in_head = False
    in_updated = False

    lines = content.splitlines(keepends=True)
    for line in lines:
        stripped = line.strip()
        if stripped == search_marker:
            in_head = True
            continue
        if stripped == divider_marker:
            in_head = False
            in_updated = True
            continue
        if stripped == replace_marker:
            in_head = False
            in_updated = False
            continue
        if in_head:
            heads.append(line)
        if in_updated:
            updates.append(line)

    head_content = "".join(heads)
    update_content = "".join(updates)

    # 多文件模式：去除末尾换行以避免重复
    if strip_trailing_newline:
        if head_content.endswith("\n"):
            head_content = head_content[:-1]
        if update_content.endswith("\n"):
            update_content = update_content[:-1]

    return SearchReplaceBlock(search=head_content, replace=update_content)


def parse_multi_file_edits(
    text: str,
    fence_open: str = DEFAULT_FENCE_OPEN,
    fence_close: str = DEFAULT_FENCE_CLOSE,
    search_marker: str = SEARCH_MARKER,
    divider_marker: str = DIVIDER_MARKER,
    replace_marker: str = REPLACE_MARKER,
) -> List[MultiFileEdit]:
    """解析多文件编辑格式，返回按文件分组的编辑列表。

    支持两种格式：
    1. two_line_mode:
        ```python
        ##File: path/to/file.py
        <<<<<<< SEARCH
        old content
        =======
        new content
        >>>>>>> REPLACE
        ```

    2. one_line_mode:
        ```python:path/to/file.py
        <<<<<<< SEARCH
        old content
        =======
        new content
        >>>>>>> REPLACE
        ```

    Args:
        text: 包含多文件编辑块的完整文本
        fence_open: 代码块开始标记
        fence_close: 代码块结束标记
        search_marker: SEARCH 标记
        divider_marker: 分隔标记
        replace_marker: REPLACE 标记

    Returns:
        按文件分组的 MultiFileEdit 列表
    """
    raw_blocks = _parse_multi_file_raw_blocks(
        text, fence_open, fence_close, search_marker
    )

    # 按路径分组
    path_to_blocks: dict[str, List[SearchReplaceBlock]] = {}
    for raw in raw_blocks:
        block = _parse_block_content(
            raw.content, search_marker, divider_marker, replace_marker
        )
        if raw.path not in path_to_blocks:
            path_to_blocks[raw.path] = []
        path_to_blocks[raw.path].append(block)

    return [
        MultiFileEdit(path=path, blocks=blocks)
        for path, blocks in path_to_blocks.items()
    ]


def parse_multi_file_edits_as_tuples(
    text: str,
    fence_open: str = DEFAULT_FENCE_OPEN,
    fence_close: str = DEFAULT_FENCE_CLOSE,
    search_marker: str = SEARCH_MARKER,
    divider_marker: str = DIVIDER_MARKER,
    replace_marker: str = REPLACE_MARKER,
) -> List[Tuple[str, str, str]]:
    """解析多文件编辑格式，返回 (path, search, replace) 元组列表（兼容旧接口）。

    注意：此函数返回扁平列表，每个块一个元组。
    """
    raw_blocks = _parse_multi_file_raw_blocks(
        text, fence_open, fence_close, search_marker
    )

    result: List[Tuple[str, str, str]] = []
    for raw in raw_blocks:
        block = _parse_block_content(
            raw.content, search_marker, divider_marker, replace_marker
        )
        result.append((raw.path, block.search, block.replace))

    return result


# ============================================================================
# 辅助函数
# ============================================================================


def find_line_numbers(content: str, text_block: str) -> Tuple[int, int]:
    """查找文本块在内容中的行号范围。

    Args:
        content: 完整文件内容
        text_block: 要查找的文本块

    Returns:
        (start_line, end_line) 行号（1-indexed），未找到返回 (-1, -1)
    """
    block_start_idx = content.find(text_block)
    if block_start_idx == -1:
        return (-1, -1)

    block_lines = text_block.splitlines()
    lines_before = content[:block_start_idx].count("\n")
    start_line = lines_before + 1
    lines_in_block = len(block_lines)
    end_line = start_line + lines_in_block - 1

    return (start_line, end_line)



"""
Tool Caller Test

测试 ToolCaller 和插件系统的基本功能。
"""

import pytest
import tempfile
from unittest.mock import Mock, MagicMock
from autocoder.common import AutoCoderArgs
from autocoder.common.v2.agent.agentic_edit_types import BaseTool, ToolResult, ExecuteCommandTool
from autocoder.common.v2.agent.agentic_edit_tools.base_tool_resolver import BaseToolResolver
try:
    from .tool_caller import ToolCaller
    from .plugins.examples.logging_plugin import LoggingPlugin
    from .plugins.examples.security_filter_plugin import SecurityFilterPlugin
except ImportError:
    # 当直接运行测试文件时的导入方式
    from tool_caller import ToolCaller
    from plugins.examples.logging_plugin import LoggingPlugin
    from plugins.examples.security_filter_plugin import SecurityFilterPlugin


class MockTool(BaseTool):
    """测试用的模拟工具"""
    value: str = "test"


class MockToolResolver(BaseToolResolver):
    """测试用的模拟工具解析器"""
    
    def resolve(self) -> ToolResult:
        # 根据工具类型返回不同的结果
        if isinstance(self.tool, ExecuteCommandTool):
            return ToolResult(
                success=True,
                message="Mock command executed successfully",
                content={"result": "mock_result", "command": self.tool.command}
            )
        elif hasattr(self.tool, 'value'):
            return ToolResult(
                success=True,
                message="Mock tool executed successfully",
                content={"result": "mock_result", "value": self.tool.value}
            )
        else:
            return ToolResult(
                success=True,
                message="Mock tool executed successfully",
                content={"result": "mock_result", "tool_type": type(self.tool).__name__}
            )


class TestToolCaller:
    """ToolCaller 测试类"""
    
    def setup_method(self):
        """测试前的设置"""
        self.mock_agent = Mock()
        self.temp_dir = tempfile.mkdtemp()
        self.args = AutoCoderArgs(source_dir=self.temp_dir)
        
        # 创建工具解析器映射
        self.tool_resolver_map = {
            MockTool: MockToolResolver,
            ExecuteCommandTool: MockToolResolver  # 简化测试
        }
        
        # 创建工具调用器
        self.tool_caller = ToolCaller(
            tool_resolver_map=self.tool_resolver_map,
            agent=self.mock_agent,
            args=self.args,
            enable_plugins=True
        )
    
    def teardown_method(self):
        """测试后的清理"""
        import shutil
        shutil.rmtree(self.temp_dir, ignore_errors=True)
    
    def test_basic_tool_execution(self):
        """测试基本工具执行"""
        tool = MockTool(value="test_value")
        result = self.tool_caller.call_tool(tool)
        
        assert result.success is True
        assert "successfully" in result.message
        assert result.content["value"] == "test_value"
    
    def test_tool_execution_without_plugins(self):
        """测试禁用插件时的工具执行"""
        self.tool_caller.set_plugins_enabled(False)
        
        tool = MockTool(value="no_plugins")
        result = self.tool_caller.call_tool(tool)
        
        assert result.success is True
        assert result.content["value"] == "no_plugins"
    
    def test_logging_plugin(self):
        """测试日志插件"""
        # 注册日志插件
        logging_plugin = LoggingPlugin(enabled=True, detailed_logging=True)
        assert self.tool_caller.register_plugin(logging_plugin) is True
        
        # 执行工具
        tool = MockTool(value="with_logging")
        result = self.tool_caller.call_tool(tool)
        
        assert result.success is True
        
        # 检查日志插件的记录
        history = logging_plugin.get_call_history()
        assert len(history) >= 2  # before 和 after 两条记录
        
        # 检查记录内容
        before_records = [h for h in history if h["phase"] == "before_execution"]
        after_records = [h for h in history if h["phase"] == "after_execution"]
        
        assert len(before_records) >= 1
        assert len(after_records) >= 1
        assert before_records[0]["tool_name"] == "MockTool"
        assert after_records[0]["success"] is True
    
    def test_security_filter_plugin(self):
        """测试安全过滤插件"""
        # 注册安全过滤插件
        security_plugin = SecurityFilterPlugin(enabled=True)
        assert self.tool_caller.register_plugin(security_plugin) is True
        
        # 测试过滤敏感命令
        dangerous_tool = ExecuteCommandTool(
            command="rm -rf /",
            requires_approval=False
        )
        result = self.tool_caller.call_tool(dangerous_tool)
        
        # 应该被过滤阻止
        assert result.success is True  # 因为被替换为安全命令
        assert security_plugin.get_filter_statistics()["blocked_count"] >= 1
        
        # 测试过滤敏感信息
        sensitive_tool = ExecuteCommandTool(
            command="echo password=secret123",
            requires_approval=False
        )
        result = self.tool_caller.call_tool(sensitive_tool)
        
        assert result.success is True
        stats = security_plugin.get_filter_statistics()
        assert stats["filtered_count"] >= 1
    
    def test_plugin_priority(self):
        """测试插件优先级"""
        # 注册两个插件
        security_plugin = SecurityFilterPlugin(enabled=True)  # HIGHEST 优先级
        logging_plugin = LoggingPlugin(enabled=True)  # LOW 优先级
        
        self.tool_caller.register_plugin(security_plugin)
        self.tool_caller.register_plugin(logging_plugin)
        
        # 获取插件状态
        status = self.tool_caller.get_plugin_status()
        plugins = status["plugins"]
        
        # 安全插件应该排在前面（优先级更高）
        assert plugins[0]["name"] == "security_filter"
        assert plugins[1]["name"] == "logging"
    
    def test_plugin_management(self):
        """测试插件管理功能"""
        logging_plugin = LoggingPlugin()
        
        # 注册插件
        assert self.tool_caller.register_plugin(logging_plugin) is True
        
        # 检查插件状态
        status = self.tool_caller.get_plugin_status()
        assert status["total_plugins"] == 1
        assert status["enabled_plugins"] == 1
        
        # 禁用插件
        assert self.tool_caller.disable_plugin("logging") is True
        status = self.tool_caller.get_plugin_status()
        assert status["enabled_plugins"] == 0
        
        # 启用插件
        assert self.tool_caller.enable_plugin("logging") is True
        status = self.tool_caller.get_plugin_status()
        assert status["enabled_plugins"] == 1
        
        # 注销插件
        assert self.tool_caller.unregister_plugin("logging") is True
        status = self.tool_caller.get_plugin_status()
        assert status["total_plugins"] == 0
    
    def test_statistics(self):
        """测试统计功能"""
        # 执行几个工具调用
        for i in range(3):
            tool = MockTool(value=f"test_{i}")
            self.tool_caller.call_tool(tool)
        
        # 检查统计信息
        stats = self.tool_caller.get_stats()
        assert stats["total_calls"] == 3
        assert stats["successful_calls"] == 3
        assert stats["failed_calls"] == 0
        assert stats["success_rate"] == 1.0
        
        # 重置统计
        self.tool_caller.reset_stats()
        stats = self.tool_caller.get_stats()
        assert stats["total_calls"] == 0
    
    def test_error_handling(self):
        """测试错误处理"""
        # 创建一个会抛出异常的解析器
        class ErrorToolResolver(BaseToolResolver):
            def resolve(self) -> ToolResult:
                raise ValueError("Test error")
        
        # 临时替换解析器
        original_resolver = self.tool_caller.tool_resolver_map[MockTool]
        self.tool_caller.tool_resolver_map[MockTool] = ErrorToolResolver
        
        try:
            tool = MockTool(value="error_test")
            result = self.tool_caller.call_tool(tool)
            
            assert result.success is False
            assert "Test error" in result.message
            
            # 检查统计
            stats = self.tool_caller.get_stats()
            assert stats["failed_calls"] > 0
            
        finally:
            # 恢复原始解析器
            self.tool_caller.tool_resolver_map[MockTool] = original_resolver


def test_integration():
    """集成测试"""
    args = AutoCoderArgs(source_dir=".")
    
    # 创建简单的工具解析器映射
    tool_resolver_map = {
        MockTool: MockToolResolver
    }
    
    # 创建工具调用器
    tool_caller = ToolCaller(
        tool_resolver_map=tool_resolver_map,
        args=args,
        enable_plugins=True
    )
    
    # 注册插件
    logging_plugin = LoggingPlugin()
    tool_caller.register_plugin(logging_plugin)
    
    # 执行工具
    tool = MockTool(value="integration_test")
    result = tool_caller.call_tool(tool)
    
    assert result.success is True
    assert logging_plugin.get_call_history()


if __name__ == "__main__":
    # 运行简单的测试
    test_integration()
    print("Integration test passed!")
    
    # 运行完整测试需要 pytest
    print("Run 'pytest test_tool_caller.py -v' for full test suite") 


import os
from typing import Optional

from rich.console import Console
from rich.table import Table
from rich.panel import Panel

from autocoder.common.international import get_message
from autocoder.common.ac_style_command_parser import create_config, parse_typed_query
from autocoder.auto_coder_runner import get_current_files


class ListFilesHandler:
    """处理文件列表显示相关的操作"""
    
    def __init__(self):
        self.console = Console()
        self._config = None
    
    def _create_config(self):
        """创建 list_files 命令的类型化配置"""
        if self._config is None:
            self._config = (create_config()
                .collect_remainder("query")  # 收集剩余参数（虽然当前不使用）
                .command("tokens")
                .max_args(0)  # 显示Token统计
                .command("simple")
                .max_args(0)  # 简单列表模式
                .build()
            )
        return self._config
    
    def handle_list_files_command(self) -> Optional[None]:
        """
        处理文件列表命令的主入口
        
        Returns:
            None: 表示处理了命令，应该返回而不继续执行
        """
        # 获取当前文件列表
        project_root = os.getcwd()
        current_files = get_current_files()
        
        if current_files:
            self._display_files_with_tokens(current_files, project_root)
        else:
            self._display_no_files()
        
        return None
    
    def _display_files_with_tokens(self, current_files: list, project_root: str) -> None:
        """显示文件列表和Token统计"""
        # Import token counting functions
        from autocoder.common.tokens import count_file_tokens, count_directory_tokens

        table = Table(
            title="Current Files", show_header=True, header_style="bold magenta"
        )
        table.add_column("File", style="green")
        table.add_column("Tokens", style="cyan", justify="right")

        total_tokens = 0

        for file_path in current_files:
            relative_path = os.path.relpath(file_path, project_root)

            # Calculate tokens based on whether it's a file or directory
            try:
                if os.path.isfile(file_path):
                    # Handle single file
                    result = count_file_tokens(file_path)
                    if result.success:
                        tokens = result.token_count
                        total_tokens += tokens
                        table.add_row(relative_path, f"{tokens:,}")
                    else:
                        table.add_row(relative_path, "[red]Error[/red]")
                elif os.path.isdir(file_path):
                    # Handle directory recursively
                    result = count_directory_tokens(
                        file_path,
                        recursive=True
                    )
                    tokens = result.total_tokens
                    total_tokens += tokens
                    table.add_row(f"{relative_path}/", f"{tokens:,}")
                else:
                    # Handle non-existent files
                    table.add_row(relative_path, "[yellow]Not found[/yellow]")
            except Exception as e:
                # Handle any errors during token counting
                table.add_row(
                    relative_path, f"[red]Error: {str(e)[:30]}[/red]")

        # Add total row
        if total_tokens > 0:
            table.add_section()
            table.add_row("[bold]Total[/bold]",
                          f"[bold cyan]{total_tokens:,}[/bold cyan]")

        self.console.print(Panel(table, border_style="blue"))
    
    def _display_no_files(self) -> None:
        """显示无文件提示"""
        self.console.print(
            Panel(
                "No files in the current session.",
                title="Current Files",
                border_style="yellow",
            )
        )



"""聊天消息模型"""

from datetime import datetime
from typing import List, Tuple
from prompt_toolkit.formatted_text import ANSI, to_formatted_text


class ChatMessage:
    """聊天消息类"""

    def __init__(
        self,
        content: str,
        role: str = "assistant",
        timestamp: datetime = None,
        ansi: bool = False,
    ):
        self.content = content
        self.role = role  # user, assistant, system, welcome
        self.timestamp = timestamp or datetime.now()
        self.is_ansi = ansi
        self._is_thinking: bool = False

    def to_formatted_text(self) -> List[Tuple[str, str]]:
        """转换为 FormattedText 格式"""
        if self.is_ansi:
            # 直接使用 ANSI 渲染（不加前缀圆点，避免破坏颜色布局）
            return list(to_formatted_text(ANSI(self.content))) + [("", "\n")]

        if self.role == "welcome":
            # 欢迎消息不显示时间戳
            return [("class:welcome", self.content), ("", "\n")]
        elif self.role == "system":
            return [
                ("class:system", "● "),
                ("class:system-text", self.content),
                ("", "\n"),
            ]
        elif self.role == "user":
            return [
                ("class:user-prompt", "> "),
                ("class:user-text", self.content),
                ("", "\n"),
            ]
        else:  # assistant
            return [
                ("class:assistant", "● "),
                ("class:assistant-text", self.content),
                ("", "\n"),
            ]



"""
规则文件解析工具

提供规则文件解析相关的实用工具函数。
"""

import os
import re
from typing import Any

import yaml
from loguru import logger

# 导入优先级目录查找器
from autocoder.common.priority_directory_finder import (
    FinderConfig,
    PriorityDirectoryFinder,
    SearchStrategy,
    ValidationMode,
    create_file_filter,
)

from ..models.rule_file import RuleFile


def parse_rule_file(file_path: str) -> RuleFile:
    """
    解析规则文件并返回结构化的Pydantic模型对象

    Args:
        file_path: 规则文件的路径

    Returns:
        RuleFile: 包含规则文件结构化内容的Pydantic模型

    Raises:
        FileNotFoundError: 当文件不存在时
        ValueError: 当文件格式无效时
    """
    if not os.path.exists(file_path):
        logger.warning(f"[RULES] 规则文件不存在 | stage=parse file_path={file_path}")
        raise FileNotFoundError(f"规则文件不存在: {file_path}")

    if not file_path.endswith(".md"):
        logger.warning(
            f"[RULES] 无效的规则文件格式，必须是.md文件 | stage=parse file_path={file_path}"
        )
        raise ValueError(f"无效的规则文件格式，必须是.md文件: {file_path}")

    try:
        with open(file_path, encoding="utf-8") as f:
            content = f.read()

        # 解析YAML头部和Markdown内容
        yaml_pattern = re.compile(r"^---\s*\n(.*?)\n---\s*\n", re.DOTALL)
        yaml_match = yaml_pattern.search(content)

        metadata = {}
        markdown_content = content

        if yaml_match:
            yaml_content = yaml_match.group(1)
            try:
                metadata = yaml.safe_load(yaml_content)
                # 移除YAML部分，仅保留Markdown内容
                markdown_content = content[yaml_match.end() :]
            except Exception as e:
                logger.warning(
                    f"[RULES] 解析规则文件YAML头部时出错，将忽略YAML头继续解析正文 | stage=parse file_path={file_path} error={e}"
                )

        # 创建并返回Pydantic模型
        rule = RuleFile(
            description=metadata.get("description", ""),
            globs=metadata.get("globs", []),
            always_apply=metadata.get("alwaysApply", False),
            content=markdown_content.strip(),
            file_path=file_path,
        )
        return rule

    except (FileNotFoundError, ValueError):
        # 重新抛出已知的异常
        raise
    except Exception as e:
        logger.error(
            f"[RULES] 解析规则文件时出现未知错误 | stage=parse file_path={file_path} exc_type={type(e).__name__} error={e}"
        )
        raise RuntimeError(f"解析规则文件失败: {file_path}") from e


def extract_yaml_metadata(content: str) -> tuple[dict[str, Any], str]:
    """
    从文件内容中提取 YAML 前置元数据和 Markdown 内容

    Args:
        content: 文件内容

    Returns:
        tuple: (元数据字典, Markdown内容)
    """
    yaml_pattern = re.compile(r"^---\s*\n(.*?)\n---\s*\n", re.DOTALL)
    yaml_match = yaml_pattern.search(content)

    metadata = {}
    markdown_content = content

    if yaml_match:
        yaml_content = yaml_match.group(1)
        try:
            metadata = yaml.safe_load(yaml_content) or {}
            # 移除YAML部分，仅保留Markdown内容
            markdown_content = content[yaml_match.end() :]
        except Exception as e:
            logger.warning(f"[RULES] 解析YAML头部时出错 | stage=parse error={e}")

    return metadata, markdown_content.strip()


def validate_rule_file_format(file_path: str) -> bool:
    """
    验证规则文件格式是否正确

    Args:
        file_path: 规则文件路径

    Returns:
        bool: 是否为有效的规则文件格式
    """
    if not os.path.exists(file_path):
        return False

    if not file_path.endswith(".md"):
        return False

    try:
        with open(file_path, encoding="utf-8") as f:
            content = f.read()

        # 检查是否可以正常解析
        metadata, markdown_content = extract_yaml_metadata(content)

        # 基本验证：确保有内容
        if not markdown_content.strip():
            return False

        return True

    except Exception:
        return False


def get_rule_directories(project_root: str) -> list[str]:
    """
    获取所有可能的规则目录路径（按优先级排序）

    Args:
        project_root: 项目根目录

    Returns:
        list[str]: 规则目录路径列表（按优先级排序）
    """
    return [
        os.path.join(project_root, ".autocoderrules"),
        os.path.join(project_root, ".auto-coder", ".autocoderrules"),
        os.path.join(project_root, ".auto-coder", "autocoderrules"),
    ]


def find_existing_rule_directories(project_root: str) -> list[str]:
    """
    使用优先级目录查找器查找实际存在且包含规则文件的目录。

    Args:
        project_root: 项目根目录

    Returns:
        list[str]: 实际存在且包含规则文件的目录路径列表
    """
    try:
        # 创建文件过滤器，只查找.md文件
        md_filter = create_file_filter(extensions=[".md"], recursive=True)

        # 创建查找器配置，使用LIST_ALL策略返回所有有效目录
        config = FinderConfig(strategy=SearchStrategy.LIST_ALL)

        # 按优先级添加目录配置
        config.add_directory(
            path=os.path.join(project_root, ".autocoderrules"),
            priority=1,
            validation_mode=ValidationMode.HAS_SPECIFIC_FILES,
            file_filter=md_filter,
            description="项目级规则目录",
        )
        config.add_directory(
            path=os.path.join(project_root, ".auto-coder", ".autocoderrules"),
            priority=2,
            validation_mode=ValidationMode.HAS_SPECIFIC_FILES,
            file_filter=md_filter,
            description="项目.auto-coder规则目录",
        )
        config.add_directory(
            path=os.path.join(project_root, ".auto-coder", "autocoderrules"),
            priority=3,
            validation_mode=ValidationMode.HAS_SPECIFIC_FILES,
            file_filter=md_filter,
            description="项目.auto-coder/autocoderrules目录",
        )

        # 执行查找
        finder = PriorityDirectoryFinder(config)
        result = finder.find_directories()

        if result.success:
            logger.info(
                f"[RULES] 使用优先级查找器找到有效规则目录 | stage=discover_dirs dir_count={len(result.selected_directories)}"
            )
            return result.selected_directories
        else:
            logger.info(
                "[RULES] 优先级查找器未找到包含规则文件的目录 | stage=discover_dirs"
            )
            return []

    except Exception as e:
        logger.warning(
            f"[RULES] 使用优先级查找器查找规则目录时出错，回退到传统方法 | stage=fallback fallback=true exc_type={type(e).__name__} error={e}"
        )
        # 回退到传统方法
        return _find_existing_rule_directories_fallback(project_root)


def _find_existing_rule_directories_fallback(project_root: str) -> list[str]:
    """
    回退到传统的目录查找方法。

    Args:
        project_root: 项目根目录

    Returns:
        list[str]: 实际存在的规则目录路径列表
    """
    logger.info(
        f"[RULES] 回退到传统的规则目录查找方法 | stage=fallback project_root={project_root}"
    )
    potential_dirs = get_rule_directories(project_root)
    existing_dirs = [d for d in potential_dirs if os.path.isdir(d)]
    logger.info(
        f"[RULES] 传统方法查找完成 | stage=fallback dir_count={len(existing_dirs)}"
    )
    return existing_dirs



"""终端应用核心类"""

import asyncio
from prompt_toolkit.formatted_text import FormattedText
from prompt_toolkit.styles import Style
from autocoder.version import __version__
from autocoder.chat_auto_coder_lang import get_message
from autocoder.common.international import get_message as get_i18n_message
from autocoder.common.global_cancel import (
    global_cancel,
    CancelRequestedException,
)
from autocoder.events.event_manager_singleton import gengerate_event_file_path
from autocoder.terminal.tasks.task_event import TaskEvent, CancellationRegistry
from autocoder.terminal.tasks.background import background_task
from autocoder.terminal.ui.completer import EnhancedCompleter
from autocoder.terminal.ui.keybindings import setup_keybindings
from autocoder.terminal.ui.toolbar import get_bottom_toolbar_func
from autocoder.terminal.ui.session import create_session
from autocoder.terminal.command_registry import CommandRegistry
from autocoder.terminal.command_processor import CommandProcessor
from autocoder.terminal.utils.errors import print_error


class TerminalApp:
    """终端应用主类"""

    def __init__(
        self,
        plugin_manager,
        wrapped_functions,
        configure_func,
        show_help_func,
        base_completer,
        get_mode_func,
        get_human_as_model_string_func,
        get_agentic_mode_string_func,
        cycle_mode_func,
        toggle_human_as_model_func,
        toggle_agentic_mode_func,
        voice_input_func,
        get_mcp_server_func,
        stop_engine_func,
        debug=False,
    ):
        self.plugin_manager = plugin_manager
        self.wrapped_functions = wrapped_functions
        self.configure = configure_func
        self.show_help = show_help_func
        self.base_completer = base_completer
        self.get_mode = get_mode_func
        self.get_human_as_model_string = get_human_as_model_string_func
        self.get_agentic_mode_string = get_agentic_mode_string_func
        self.cycle_mode = cycle_mode_func
        self.toggle_human_as_model = toggle_human_as_model_func
        self.toggle_agentic_mode = toggle_agentic_mode_func
        self.voice_input = voice_input_func
        self.get_mcp_server = get_mcp_server_func
        self.stop_engine = stop_engine_func
        self.debug = debug

        # 初始化组件
        self.task_event = TaskEvent()
        self.registry = CancellationRegistry()
        self.input_queue = asyncio.Queue()
        self.busy_event = asyncio.Event()
        self.busy_event.set()  # 初始状态为空闲

        # 创建命令处理器
        self.command_processor = CommandProcessor(
            plugin_manager=plugin_manager,
            wrapped_functions=wrapped_functions,
            configure_func=configure_func,
            global_cancel=global_cancel,
            show_help_func=show_help_func,
            debug=debug,
        )

        # 创建命令注册表
        self.command_registry = CommandRegistry()
        self._register_commands()

        # UI 组件（稍后初始化）
        self.session = None
        self.consumer_task = None
        self.background_task_coro = None
        self.stop_ev = asyncio.Event()

    def _register_commands(self):
        """注册所有命令"""
        # 注意：按前缀长度排序，优先匹配更长的前缀
        self.command_registry.register(
            "/conf/export", self.command_processor.handle_conf_export
        )
        self.command_registry.register(
            "/index/query", self.command_processor.handle_index_query
        )
        self.command_registry.register(
            "/index/build", self.command_processor.handle_index_build
        )
        self.command_registry.register(
            "/index/export", self.command_processor.handle_index_export
        )
        self.command_registry.register(
            "/index/import", self.command_processor.handle_index_import
        )

        self.command_registry.register(
            "/voice_input", self.command_processor.handle_voice_input_command
        )
        self.command_registry.register("/clear", self.command_processor.handle_clear)
        self.command_registry.register("/cls", self.command_processor.handle_clear)
        self.command_registry.register(
            "/add_files", self.command_processor.handle_add_files
        )
        self.command_registry.register(
            "/remove_files", self.command_processor.handle_remove_files
        )
        self.command_registry.register(
            "/list_files", self.command_processor.handle_list_files
        )
        self.command_registry.register("/models", self.command_processor.handle_models)
        self.command_registry.register("/mode", self.command_processor.handle_mode)
        self.command_registry.register(
            "/plugins", self.command_processor.handle_plugins
        )
        self.command_registry.register("/conf", self.command_processor.handle_conf)
        self.command_registry.register("/revert", self.command_processor.handle_revert)
        self.command_registry.register("/commit", self.command_processor.handle_commit)
        self.command_registry.register("/help", self.command_processor.handle_help)
        self.command_registry.register(
            "/exclude_dirs", self.command_processor.handle_exclude_dirs
        )
        self.command_registry.register(
            "/exclude_files", self.command_processor.handle_exclude_files
        )
        self.command_registry.register("/exit", self.command_processor.handle_exit)
        self.command_registry.register("/coding", self.command_processor.handle_coding)
        self.command_registry.register("/chat", self.command_processor.handle_chat)
        self.command_registry.register("/design", self.command_processor.handle_design)
        self.command_registry.register("/summon", self.command_processor.handle_summon)
        self.command_registry.register("/lib", self.command_processor.handle_lib)
        self.command_registry.register("/rules", self.command_processor.handle_rules)
        self.command_registry.register("/mcp", self.command_processor.handle_mcp)
        self.command_registry.register(
            "/active_context", self.command_processor.handle_active_context
        )
        self.command_registry.register("/auto", self.command_processor.handle_auto)
        self.command_registry.register("/debug", self.command_processor.handle_debug)
        self.command_registry.register(
            "/shell", self.command_processor.handle_shell_command
        )
        self.command_registry.register(
            "/workflow", self.command_processor.handle_workflow
        )
        self.command_registry.register("/remote", self.command_processor.handle_remote)
        self.command_registry.register("/rags", self.command_processor.handle_rags)

        # 兜底处理器
        self.command_registry.set_fallback(
            self.command_processor.handle_unknown_or_fallback
        )

    async def process_user_input(
        self, user_input: str, new_prompt_callback, session=None
    ):
        """处理用户输入的异步函数"""
        try:
            # 预处理输入
            user_input = self.command_processor.preprocess_input(user_input)

            # 1. 如果用户输入 /shell（无参数），启动一个子 shell
            if user_input == "/shell":
                await self.command_processor.handle_shell_enter(
                    user_input, {"session": session}
                )
                return

            # 2. 如果以 ! 开头，当作单行 shell 命令执行
            if user_input.startswith("!"):
                await self.command_processor.handle_shell_single(
                    user_input, {"session": session}
                )
                return

            # 3. 如果以 $ 开头，当作 workflow 快捷命令执行
            if user_input.startswith("$"):
                await self.command_processor.handle_workflow_shortcut(
                    user_input, {"session": session}
                )
                return

            # 4. 尝试插件处理
            context = {"session": session, "new_prompt_callback": new_prompt_callback}
            if self.command_processor.handle_plugin_command(user_input, context):
                return

            # 5. Shell 模式处理
            if await self.command_processor.handle_shell_mode(user_input, context):
                return

            # 6. 自动检测模式
            if self.command_processor.handle_auto_detect_mode(user_input, context):
                return

            # 7. 语音输入模式
            if self.command_processor.handle_voice_input_mode(user_input, context):
                return

            # 8. 使用命令注册表分发命令
            await self.command_registry.dispatch(user_input, context)

        except EOFError:
            # 重新抛出这些异常，让主循环处理
            raise
        except asyncio.CancelledError:
            # 原生取消，向上抛出让上层统一处理
            raise
        except (CancelRequestedException, KeyboardInterrupt):
            pass
        except Exception as e:
            print_error("1", e, self.debug)

    async def consumer_loop(self):
        """后台处理用户输入的消费者协程"""
        while True:
            is_queue_geted = False
            event_file = None
            try:
                # 给队列获取操作添加超时，让消费者有机会响应取消请求
                try:
                    user_input, new_prompt_callback, session = await asyncio.wait_for(
                        self.input_queue.get(), timeout=1.0
                    )
                    is_queue_geted = True
                except asyncio.TimeoutError:
                    # 超时后短暂休眠，然后继续循环等待
                    await asyncio.sleep(0.1)
                    continue

                # 设置任务状态为已开始
                self.task_event.set_state("started")
                self.busy_event.clear()  # 标记为忙碌

                # 设置任务状态为运行中
                self.task_event.set_state("running")

                # 为每个请求创建 Task 并注册（scope 自动管理 register/unregister）
                event_file, _ = gengerate_event_file_path()

                with global_cancel.scope(event_file):
                    task = asyncio.create_task(
                        self.process_user_input(
                            user_input, new_prompt_callback, session
                        )
                    )
                    self.registry.register(event_file, task)

                    try:
                        await task
                    except asyncio.CancelledError:
                        # Task 被取消，向上传播
                        raise
                    except CancelRequestedException:
                        # 兼容旧的取消机制
                        pass
                    finally:
                        self.registry.unregister(event_file)

                # 触发任务完成回调（虽然当前为空，但保持与原代码一致）
                # await trigger_task_completion_callbacks()

                # 设置任务状态为已完成
                self.task_event.set_state("completed")

            except KeyboardInterrupt:
                # 消费者收到 KeyboardInterrupt（不太可能直接收到，但保持兼容）
                # 设置取消标志并清理状态
                global_cancel.set_active_tokens()
                self.task_event.set_state("completed")
                continue
            except asyncio.CancelledError:
                break
            except Exception as e:
                if self.debug:
                    import traceback

                    traceback.print_exc()
                continue
            finally:
                self.busy_event.set()  # 标记为空闲
                if is_queue_geted:
                    self.input_queue.task_done()
                # 确保任务状态最终被重置
                if self.task_event.get_state() != "completed":
                    self.task_event.set_state("completed")

    async def run(self):
        """运行终端应用"""
        # 创建 UI 组件
        kb = setup_keybindings(
            self.plugin_manager,
            self.voice_input,
            self.cycle_mode,
            self.toggle_human_as_model,
            self.toggle_agentic_mode,
            self.configure,
        )

        toolbar_func = get_bottom_toolbar_func(
            self.get_mode,
            self.get_human_as_model_string,
            self.get_agentic_mode_string,
            self.plugin_manager,
        )

        enhanced_completer = EnhancedCompleter(self.base_completer, self.plugin_manager)

        self.session = create_session(enhanced_completer, kb, toolbar_func)

        # 启动消费者协程
        self.consumer_task = asyncio.create_task(self.consumer_loop())

        # 启动后台任务
        self.background_task_coro = asyncio.create_task(
            background_task(self.stop_ev, self.session, self.debug)
        )

        # 样式定义
        style = Style.from_dict(
            {
                "username": "#884444",
                "at": "#00aa00",
                "colon": "#0000aa",
                "pound": "#00aa00",
                "host": "#00ffff bg:#444400",
                "busy": "#ff6600 italic",
            }
        )

        # 显示启动信息
        print(
            f"""
    \033[1;32m  ____ _           _          _         _               ____          _           
    / ___| |__   __ _| |_       / \\  _   _| |_ ___        / ___|___   __| | ___ _ __ 
    | |   | '_ \\ / _` | __|____ / _ \\| | | | __/ _ \\ _____| |   / _ \\ / _` |/ _ \\ '__|
    | |___| | | | (_| | ||_____/ ___ \\ |_| | || (_) |_____| |__| (_) | (_| |  __/ |   
    \\____|_| |_|\\__,_|\\__|   /_/   \\_\\__,_|\\__\\___/       \\____\\___/ \\__,_|\\___|_| 
                                                                        v{__version__}
    \033[0m"""
        )
        print(f"\033[1;34m{get_message('type_help_to_see_commands')}\033[0m\n")

        # 显示插件信息
        if self.plugin_manager.plugins:
            print(f"\033[1;34m{get_message('loaded_plugins_title')}\033[0m")
            for name, plugin in self.plugin_manager.plugins.items():
                print(f"  - {name} (v{plugin.version}): {plugin.description}")
            print()

        self.show_help()

        # 用于设置新提示的回调函数
        new_prompt = ""

        def set_new_prompt(prompt):
            nonlocal new_prompt
            new_prompt = prompt

        # 主交互循环
        while True:
            task_state = self.task_event.get_state()
            try:
                # 根据任务状态控制是否渲染prompt
                if task_state in ["started", "running", "pending"]:
                    # 任务运行中，不显示prompt，跳过此次循环
                    # 使用 sleep 让循环有机会响应 Ctrl+C
                    await asyncio.sleep(0.1)
                    continue
                else:
                    prompt_message = [
                        ("class:username", "coding"),
                        ("class:at", "@"),
                        ("class:host", "auto-coder.chat"),
                        ("class:colon", ":"),
                        ("class:path", "~"),
                        ("class:dollar", "$ "),
                    ]

                if new_prompt:
                    user_input = await self.session.prompt_async(
                        FormattedText(prompt_message), default=new_prompt, style=style
                    )
                    new_prompt = ""
                else:
                    user_input = await self.session.prompt_async(
                        FormattedText(prompt_message), style=style
                    )

                if user_input.strip() == "/exit":
                    raise EOFError()

                # 将输入放入队列
                if user_input.strip():
                    await self.input_queue.put(
                        (user_input, set_new_prompt, self.session)
                    )
                    self.task_event.set_state("pending")

            except (KeyboardInterrupt, asyncio.CancelledError):
                # 按 Ctrl+C 取消当前任务
                global_cancel.set_active_tokens()
                continue
            except CancelRequestedException:
                continue
            except EOFError:
                break
            except Exception as e:
                print_error("2", e, self.debug)

        # 清理
        await self.shutdown()

    async def shutdown(self):
        """关闭应用并清理资源"""
        exit_msg = get_i18n_message("exit_ctrl_d")
        print(f"\n\033[93m{exit_msg}\033[0m")

        # 通知后台任务停止
        self.stop_ev.set()

        # 取消消费者任务
        if self.consumer_task:
            self.consumer_task.cancel()
            try:
                await self.consumer_task
            except asyncio.CancelledError:
                pass

        # 取消后台任务
        if self.background_task_coro:
            self.background_task_coro.cancel()
            try:
                await self.background_task_coro
            except asyncio.CancelledError:
                pass

        # 清理资源
        try:
            # 关闭所有插件
            self.plugin_manager.shutdown_all()
            # 停止 MCP 服务器
            try:
                if self.get_mcp_server():
                    self.get_mcp_server().stop()
            except Exception:
                pass
            # 停止引擎
            self.stop_engine()
        except Exception as e:
            print_error("3 (cleanup)", e, self.debug)

        goodbye_msg = get_i18n_message("goodbye")
        print(f"\n\033[93m{goodbye_msg}\033[0m")
